# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

import warnings

warnings.warn("dazl.protocols.v0 is deprecated", DeprecationWarning, stacklevel=2)
