# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/daml_lf_dev/daml_lf_1.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/daml/daml_lf_dev/daml_lf_1.proto',
  package='daml_lf_1',
  syntax='proto3',
  serialized_options=b'\n\024com.daml.daml_lf_dev\252\002\034Com.Daml.Daml_Lf_Dev.DamlLf1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n$com/daml/daml_lf_dev/daml_lf_1.proto\x12\tdaml_lf_1\"\x06\n\x04Unit\"q\n\nPackageRef\x12\x1f\n\x04self\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.UnitH\x00\x12\x18\n\x0epackage_id_str\x18\x02 \x01(\tH\x00\x12!\n\x17package_id_interned_str\x18\x03 \x01(\x05H\x00\x42\x05\n\x03Sum\"\x1e\n\nDottedName\x12\x10\n\x08segments\x18\x01 \x03(\t\"\xa0\x01\n\tModuleRef\x12*\n\x0bpackage_ref\x18\x01 \x01(\x0b\x32\x15.daml_lf_1.PackageRef\x12\x32\n\x11module_name_dname\x18\x02 \x01(\x0b\x32\x15.daml_lf_1.DottedNameH\x00\x12$\n\x1amodule_name_interned_dname\x18\x03 \x01(\x05H\x00\x42\r\n\x0bmodule_name\"\x87\x01\n\x0bTypeConName\x12$\n\x06module\x18\x01 \x01(\x0b\x32\x14.daml_lf_1.ModuleRef\x12+\n\nname_dname\x18\x02 \x01(\x0b\x32\x15.daml_lf_1.DottedNameH\x00\x12\x1d\n\x13name_interned_dname\x18\x03 \x01(\x05H\x00\x42\x06\n\x04name\"\x87\x01\n\x0bTypeSynName\x12$\n\x06module\x18\x01 \x01(\x0b\x32\x14.daml_lf_1.ModuleRef\x12+\n\nname_dname\x18\x02 \x01(\x0b\x32\x15.daml_lf_1.DottedNameH\x00\x12\x1d\n\x13name_interned_dname\x18\x03 \x01(\x05H\x00\x42\x06\n\x04name\"`\n\x07ValName\x12$\n\x06module\x18\x01 \x01(\x0b\x32\x14.daml_lf_1.ModuleRef\x12\x12\n\nname_dname\x18\x02 \x03(\t\x12\x1b\n\x13name_interned_dname\x18\x03 \x01(\x05\"j\n\rFieldWithType\x12\x13\n\tfield_str\x18\x01 \x01(\tH\x00\x12\x1c\n\x12\x66ield_interned_str\x18\x03 \x01(\x05H\x00\x12\x1d\n\x04type\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.TypeB\x07\n\x05\x66ield\"b\n\x0bVarWithType\x12\x11\n\x07var_str\x18\x01 \x01(\tH\x00\x12\x1a\n\x10var_interned_str\x18\x03 \x01(\x05H\x00\x12\x1d\n\x04type\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.TypeB\x05\n\x03var\"f\n\x0fTypeVarWithKind\x12\x11\n\x07var_str\x18\x01 \x01(\tH\x00\x12\x1a\n\x10var_interned_str\x18\x03 \x01(\x05H\x00\x12\x1d\n\x04kind\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.KindB\x05\n\x03var\"j\n\rFieldWithExpr\x12\x13\n\tfield_str\x18\x01 \x01(\tH\x00\x12\x1c\n\x12\x66ield_interned_str\x18\x03 \x01(\x05H\x00\x12\x1d\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\x07\n\x05\x66ield\"Q\n\x07\x42inding\x12&\n\x06\x62inder\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.VarWithType\x12\x1e\n\x05\x62ound\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\"\xc1\x01\n\x04Kind\x12\x1f\n\x04star\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.UnitH\x00\x12&\n\x05\x61rrow\x18\x02 \x01(\x0b\x32\x15.daml_lf_1.Kind.ArrowH\x00\x12\x1e\n\x03nat\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.UnitH\x00\x1aI\n\x05\x41rrow\x12\x1f\n\x06params\x18\x01 \x03(\x0b\x32\x0f.daml_lf_1.Kind\x12\x1f\n\x06result\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.KindB\x05\n\x03Sum\"\xc4\x06\n\x04Type\x12\"\n\x03var\x18\x01 \x01(\x0b\x32\x13.daml_lf_1.Type.VarH\x00\x12\"\n\x03\x63on\x18\x02 \x01(\x0b\x32\x13.daml_lf_1.Type.ConH\x00\x12$\n\x04prim\x18\x03 \x01(\x0b\x32\x14.daml_lf_1.Type.PrimH\x00\x12(\n\x06\x66orall\x18\x05 \x01(\x0b\x32\x16.daml_lf_1.Type.ForallH\x00\x12(\n\x06struct\x18\x07 \x01(\x0b\x32\x16.daml_lf_1.Type.StructH\x00\x12\r\n\x03nat\x18\x0b \x01(\x12H\x00\x12\"\n\x03syn\x18\x0c \x01(\x0b\x32\x13.daml_lf_1.Type.SynH\x00\x12\x12\n\x08interned\x18\r \x01(\x05H\x00\x1aZ\n\x03Var\x12\x11\n\x07var_str\x18\x01 \x01(\tH\x00\x12\x1a\n\x10var_interned_str\x18\x03 \x01(\x05H\x00\x12\x1d\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x0f.daml_lf_1.TypeB\x05\n\x03var\x1aK\n\x03\x43on\x12%\n\x05tycon\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeConName\x12\x1d\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x0f.daml_lf_1.Type\x1aK\n\x03Syn\x12%\n\x05tysyn\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeSynName\x12\x1d\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x0f.daml_lf_1.Type\x1aH\n\x04Prim\x12!\n\x04prim\x18\x01 \x01(\x0e\x32\x13.daml_lf_1.PrimType\x12\x1d\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x0f.daml_lf_1.Type\x1aG\n\x03\x46un\x12\x1f\n\x06params\x18\x01 \x03(\x0b\x32\x0f.daml_lf_1.Type\x12\x1f\n\x06result\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Type\x1aQ\n\x06\x46orall\x12(\n\x04vars\x18\x01 \x03(\x0b\x32\x1a.daml_lf_1.TypeVarWithKind\x12\x1d\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Type\x1a\x32\n\x06Struct\x12(\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x18.daml_lf_1.FieldWithTypeB\x05\n\x03SumJ\x04\x08\x04\x10\x05J\x04\x08\x06\x10\x07J\x04\x08\x08\x10\tJ\x04\x08\t\x10\nJ\x04\x08\n\x10\x0b\"\xed\x01\n\x07PrimLit\x12\x0f\n\x05int64\x18\x01 \x01(\x12H\x00\x12\x15\n\x0b\x64\x65\x63imal_str\x18\x02 \x01(\tH\x00\x12\x1e\n\x14numeric_interned_str\x18\n \x01(\x05H\x00\x12\x12\n\x08text_str\x18\x04 \x01(\tH\x00\x12\x1b\n\x11text_interned_str\x18\x0b \x01(\x05H\x00\x12\x13\n\ttimestamp\x18\x05 \x01(\x10H\x00\x12\x13\n\tparty_str\x18\x07 \x01(\tH\x00\x12\x1c\n\x12party_interned_str\x18\x0c \x01(\x05H\x00\x12\x0e\n\x04\x64\x61te\x18\x08 \x01(\x05H\x00\x42\x05\n\x03SumJ\x04\x08\x03\x10\x04J\x04\x08\x06\x10\x07\"\xad\x01\n\x08Location\x12$\n\x06module\x18\x01 \x01(\x0b\x32\x14.daml_lf_1.ModuleRef\x12(\n\x05range\x18\x02 \x01(\x0b\x32\x19.daml_lf_1.Location.Range\x1aQ\n\x05Range\x12\x12\n\nstart_line\x18\x01 \x01(\x05\x12\x11\n\tstart_col\x18\x02 \x01(\x05\x12\x10\n\x08\x65nd_line\x18\x03 \x01(\x05\x12\x0f\n\x07\x65nd_col\x18\x04 \x01(\x05\"\xd0\x1a\n\x04\x45xpr\x12%\n\x08location\x18\x19 \x01(\x0b\x32\x13.daml_lf_1.Location\x12\x11\n\x07var_str\x18\x01 \x01(\tH\x00\x12\x1a\n\x10var_interned_str\x18\x1d \x01(\x05H\x00\x12!\n\x03val\x18\x02 \x01(\x0b\x32\x12.daml_lf_1.ValNameH\x00\x12-\n\x07\x62uiltin\x18\x03 \x01(\x0e\x32\x1a.daml_lf_1.BuiltinFunctionH\x00\x12&\n\x08prim_con\x18\x04 \x01(\x0e\x32\x12.daml_lf_1.PrimConH\x00\x12&\n\x08prim_lit\x18\x05 \x01(\x0b\x32\x12.daml_lf_1.PrimLitH\x00\x12)\n\x07rec_con\x18\x06 \x01(\x0b\x32\x16.daml_lf_1.Expr.RecConH\x00\x12+\n\x08rec_proj\x18\x07 \x01(\x0b\x32\x17.daml_lf_1.Expr.RecProjH\x00\x12)\n\x07rec_upd\x18\x16 \x01(\x0b\x32\x16.daml_lf_1.Expr.RecUpdH\x00\x12\x31\n\x0bvariant_con\x18\x08 \x01(\x0b\x32\x1a.daml_lf_1.Expr.VariantConH\x00\x12+\n\x08\x65num_con\x18\x1c \x01(\x0b\x32\x17.daml_lf_1.Expr.EnumConH\x00\x12/\n\nstruct_con\x18\t \x01(\x0b\x32\x19.daml_lf_1.Expr.StructConH\x00\x12\x31\n\x0bstruct_proj\x18\n \x01(\x0b\x32\x1a.daml_lf_1.Expr.StructProjH\x00\x12/\n\nstruct_upd\x18\x17 \x01(\x0b\x32\x19.daml_lf_1.Expr.StructUpdH\x00\x12\"\n\x03\x61pp\x18\x0b \x01(\x0b\x32\x13.daml_lf_1.Expr.AppH\x00\x12\'\n\x06ty_app\x18\x0c \x01(\x0b\x32\x15.daml_lf_1.Expr.TyAppH\x00\x12\"\n\x03\x61\x62s\x18\r \x01(\x0b\x32\x13.daml_lf_1.Expr.AbsH\x00\x12\'\n\x06ty_abs\x18\x0e \x01(\x0b\x32\x15.daml_lf_1.Expr.TyAbsH\x00\x12\x1f\n\x04\x63\x61se\x18\x0f \x01(\x0b\x32\x0f.daml_lf_1.CaseH\x00\x12\x1f\n\x03let\x18\x10 \x01(\x0b\x32\x10.daml_lf_1.BlockH\x00\x12\"\n\x03nil\x18\x11 \x01(\x0b\x32\x13.daml_lf_1.Expr.NilH\x00\x12$\n\x04\x63ons\x18\x12 \x01(\x0b\x32\x14.daml_lf_1.Expr.ConsH\x00\x12#\n\x06update\x18\x14 \x01(\x0b\x32\x11.daml_lf_1.UpdateH\x00\x12\'\n\x08scenario\x18\x15 \x01(\x0b\x32\x13.daml_lf_1.ScenarioH\x00\x12\x35\n\roptional_none\x18\x1a \x01(\x0b\x32\x1c.daml_lf_1.Expr.OptionalNoneH\x00\x12\x35\n\roptional_some\x18\x1b \x01(\x0b\x32\x1c.daml_lf_1.Expr.OptionalSomeH\x00\x12\'\n\x06to_any\x18\x1e \x01(\x0b\x32\x15.daml_lf_1.Expr.ToAnyH\x00\x12+\n\x08\x66rom_any\x18\x1f \x01(\x0b\x32\x17.daml_lf_1.Expr.FromAnyH\x00\x12#\n\x08type_rep\x18  \x01(\x0b\x32\x0f.daml_lf_1.TypeH\x00\x12:\n\x10to_any_exception\x18! \x01(\x0b\x32\x1e.daml_lf_1.Expr.ToAnyExceptionH\x00\x12>\n\x12\x66rom_any_exception\x18\" \x01(\x0b\x32 .daml_lf_1.Expr.FromAnyExceptionH\x00\x12&\n\x05throw\x18# \x01(\x0b\x32\x15.daml_lf_1.Expr.ThrowH\x00\x1aV\n\x06RecCon\x12\"\n\x05tycon\x18\x01 \x01(\x0b\x32\x13.daml_lf_1.Type.Con\x12(\n\x06\x66ields\x18\x02 \x03(\x0b\x32\x18.daml_lf_1.FieldWithExpr\x1a\x8a\x01\n\x07RecProj\x12\"\n\x05tycon\x18\x01 \x01(\x0b\x32\x13.daml_lf_1.Type.Con\x12\x13\n\tfield_str\x18\x02 \x01(\tH\x00\x12\x1c\n\x12\x66ield_interned_str\x18\x04 \x01(\x05H\x00\x12\x1f\n\x06record\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\x07\n\x05\x66ield\x1a\xaa\x01\n\x06RecUpd\x12\"\n\x05tycon\x18\x01 \x01(\x0b\x32\x13.daml_lf_1.Type.Con\x12\x13\n\tfield_str\x18\x02 \x01(\tH\x00\x12\x1c\n\x12\x66ield_interned_str\x18\x05 \x01(\x05H\x00\x12\x1f\n\x06record\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x1f\n\x06update\x18\x04 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\x07\n\x05\x66ield\x1a\xa4\x01\n\nVariantCon\x12\"\n\x05tycon\x18\x01 \x01(\x0b\x32\x13.daml_lf_1.Type.Con\x12\x19\n\x0fvariant_con_str\x18\x02 \x01(\tH\x00\x12\"\n\x18variant_con_interned_str\x18\x04 \x01(\x05H\x00\x12$\n\x0bvariant_arg\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\r\n\x0bvariant_con\x1au\n\x07\x45numCon\x12%\n\x05tycon\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeConName\x12\x16\n\x0c\x65num_con_str\x18\x02 \x01(\tH\x00\x12\x1f\n\x15\x65num_con_interned_str\x18\x03 \x01(\x05H\x00\x42\n\n\x08\x65num_con\x1a\x35\n\tStructCon\x12(\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x18.daml_lf_1.FieldWithExpr\x1ai\n\nStructProj\x12\x13\n\tfield_str\x18\x01 \x01(\tH\x00\x12\x1c\n\x12\x66ield_interned_str\x18\x03 \x01(\x05H\x00\x12\x1f\n\x06struct\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\x07\n\x05\x66ield\x1a\x89\x01\n\tStructUpd\x12\x13\n\tfield_str\x18\x01 \x01(\tH\x00\x12\x1c\n\x12\x66ield_interned_str\x18\x04 \x01(\x05H\x00\x12\x1f\n\x06struct\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x1f\n\x06update\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\x07\n\x05\x66ield\x1a\x42\n\x03\x41pp\x12\x1c\n\x03\x66un\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x1d\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x0f.daml_lf_1.Expr\x1a\x46\n\x05TyApp\x12\x1d\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x1e\n\x05types\x18\x02 \x03(\x0b\x32\x0f.daml_lf_1.Type\x1aK\n\x03\x41\x62s\x12%\n\x05param\x18\x01 \x03(\x0b\x32\x16.daml_lf_1.VarWithType\x12\x1d\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1aQ\n\x05TyAbs\x12)\n\x05param\x18\x01 \x03(\x0b\x32\x1a.daml_lf_1.TypeVarWithKind\x12\x1d\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1a$\n\x03Nil\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x1a\x64\n\x04\x43ons\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1e\n\x05\x66ront\x18\x02 \x03(\x0b\x32\x0f.daml_lf_1.Expr\x12\x1d\n\x04tail\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1a-\n\x0cOptionalNone\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x1aL\n\x0cOptionalSome\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1d\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1a\x45\n\x05ToAny\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1d\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1aG\n\x07\x46romAny\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1d\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1aN\n\x0eToAnyException\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1d\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1aP\n\x10\x46romAnyException\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1d\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1a\x7f\n\x05Throw\x12$\n\x0breturn_type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\'\n\x0e\x65xception_type\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\'\n\x0e\x65xception_expr\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\x05\n\x03SumJ\x04\x08\x13\x10\x14J\x04\x08\x18\x10\x19\"\x95\x07\n\x07\x43\x61seAlt\x12\"\n\x07\x64\x65\x66\x61ult\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.UnitH\x00\x12-\n\x07variant\x18\x02 \x01(\x0b\x32\x1a.daml_lf_1.CaseAlt.VariantH\x00\x12&\n\x08prim_con\x18\x03 \x01(\x0e\x32\x12.daml_lf_1.PrimConH\x00\x12\x1e\n\x03nil\x18\x04 \x01(\x0b\x32\x0f.daml_lf_1.UnitH\x00\x12\'\n\x04\x63ons\x18\x05 \x01(\x0b\x32\x17.daml_lf_1.CaseAlt.ConsH\x00\x12(\n\roptional_none\x18\x07 \x01(\x0b\x32\x0f.daml_lf_1.UnitH\x00\x12\x38\n\roptional_some\x18\x08 \x01(\x0b\x32\x1f.daml_lf_1.CaseAlt.OptionalSomeH\x00\x12\'\n\x04\x65num\x18\t \x01(\x0b\x32\x17.daml_lf_1.CaseAlt.EnumH\x00\x12\x1d\n\x04\x62ody\x18\x06 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1a\xaf\x01\n\x07Variant\x12#\n\x03\x63on\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeConName\x12\x15\n\x0bvariant_str\x18\x02 \x01(\tH\x00\x12\x1e\n\x14variant_interned_str\x18\x04 \x01(\x05H\x00\x12\x14\n\nbinder_str\x18\x03 \x01(\tH\x01\x12\x1d\n\x13\x62inder_interned_str\x18\x05 \x01(\x05H\x01\x42\t\n\x07variantB\x08\n\x06\x62inder\x1ay\n\x04\x45num\x12#\n\x03\x63on\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeConName\x12\x19\n\x0f\x63onstructor_str\x18\x02 \x01(\tH\x00\x12\"\n\x18\x63onstructor_interned_str\x18\x03 \x01(\x05H\x00\x42\r\n\x0b\x63onstructor\x1a\x90\x01\n\x04\x43ons\x12\x16\n\x0cvar_head_str\x18\x01 \x01(\tH\x00\x12\x1f\n\x15var_head_interned_str\x18\x03 \x01(\x05H\x00\x12\x16\n\x0cvar_tail_str\x18\x02 \x01(\tH\x01\x12\x1f\n\x15var_tail_interned_str\x18\x04 \x01(\x05H\x01\x42\n\n\x08var_headB\n\n\x08var_tail\x1aS\n\x0cOptionalSome\x12\x16\n\x0cvar_body_str\x18\x01 \x01(\tH\x00\x12\x1f\n\x15var_body_interned_str\x18\x02 \x01(\x05H\x00\x42\n\n\x08var_bodyB\x05\n\x03Sum\"H\n\x04\x43\x61se\x12\x1e\n\x05scrut\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12 \n\x04\x61lts\x18\x02 \x03(\x0b\x32\x12.daml_lf_1.CaseAlt\"L\n\x05\x42lock\x12$\n\x08\x62indings\x18\x01 \x03(\x0b\x32\x12.daml_lf_1.Binding\x12\x1d\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\"D\n\x04Pure\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1d\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\"\xc1\n\n\x06Update\x12\x1f\n\x04pure\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.PureH\x00\x12!\n\x05\x62lock\x18\x02 \x01(\x0b\x32\x10.daml_lf_1.BlockH\x00\x12*\n\x06\x63reate\x18\x03 \x01(\x0b\x32\x18.daml_lf_1.Update.CreateH\x00\x12.\n\x08\x65xercise\x18\x04 \x01(\x0b\x32\x1a.daml_lf_1.Update.ExerciseH\x00\x12:\n\x0f\x65xercise_by_key\x18\n \x01(\x0b\x32\x1f.daml_lf_1.Update.ExerciseByKeyH\x00\x12(\n\x05\x66\x65tch\x18\x05 \x01(\x0b\x32\x17.daml_lf_1.Update.FetchH\x00\x12#\n\x08get_time\x18\x06 \x01(\x0b\x32\x0f.daml_lf_1.UnitH\x00\x12\x38\n\rlookup_by_key\x18\x08 \x01(\x0b\x32\x1f.daml_lf_1.Update.RetrieveByKeyH\x00\x12\x37\n\x0c\x66\x65tch_by_key\x18\t \x01(\x0b\x32\x1f.daml_lf_1.Update.RetrieveByKeyH\x00\x12\x31\n\nembed_expr\x18\x07 \x01(\x0b\x32\x1b.daml_lf_1.Update.EmbedExprH\x00\x12/\n\ttry_catch\x18\x0b \x01(\x0b\x32\x1a.daml_lf_1.Update.TryCatchH\x00\x1aQ\n\x06\x43reate\x12(\n\x08template\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeConName\x12\x1d\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1a\xb5\x01\n\x08\x45xercise\x12(\n\x08template\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeConName\x12\x14\n\nchoice_str\x18\x02 \x01(\tH\x00\x12\x1d\n\x13\x63hoice_interned_str\x18\x06 \x01(\x05H\x00\x12\x1c\n\x03\x63id\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x1c\n\x03\x61rg\x18\x05 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\x08\n\x06\x63hoiceJ\x04\x08\x04\x10\x05\x1a\x92\x01\n\rExerciseByKey\x12(\n\x08template\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeConName\x12\x1b\n\x13\x63hoice_interned_str\x18\x02 \x01(\x05\x12\x1c\n\x03key\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x1c\n\x03\x61rg\x18\x04 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1aU\n\x05\x46\x65tch\x12(\n\x08template\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeConName\x12\x1c\n\x03\x63id\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.ExprJ\x04\x08\x03\x10\x04\x1aI\n\tEmbedExpr\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1d\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1aW\n\rRetrieveByKey\x12(\n\x08template\x18\x01 \x01(\x0b\x32\x16.daml_lf_1.TypeConName\x12\x1c\n\x03key\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x1a\x92\x01\n\x08TryCatch\x12$\n\x0breturn_type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12!\n\x08try_expr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x18\n\x10var_interned_str\x18\x03 \x01(\x05\x12#\n\ncatch_expr\x18\x04 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\x05\n\x03Sum\"\x8d\x04\n\x08Scenario\x12\x1f\n\x04pure\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.PureH\x00\x12!\n\x05\x62lock\x18\x02 \x01(\x0b\x32\x10.daml_lf_1.BlockH\x00\x12,\n\x06\x63ommit\x18\x03 \x01(\x0b\x32\x1a.daml_lf_1.Scenario.CommitH\x00\x12\x30\n\nmustFailAt\x18\x04 \x01(\x0b\x32\x1a.daml_lf_1.Scenario.CommitH\x00\x12\x1f\n\x04pass\x18\x05 \x01(\x0b\x32\x0f.daml_lf_1.ExprH\x00\x12#\n\x08get_time\x18\x06 \x01(\x0b\x32\x0f.daml_lf_1.UnitH\x00\x12$\n\tget_party\x18\x07 \x01(\x0b\x32\x0f.daml_lf_1.ExprH\x00\x12\x33\n\nembed_expr\x18\x08 \x01(\x0b\x32\x1d.daml_lf_1.Scenario.EmbedExprH\x00\x1aj\n\x06\x43ommit\x12\x1e\n\x05party\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x1d\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12!\n\x08ret_type\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.Type\x1aI\n\tEmbedExpr\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1d\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\x05\n\x03Sum\"\x8b\x03\n\x0eTemplateChoice\x12\x12\n\x08name_str\x18\x01 \x01(\tH\x00\x12\x1b\n\x11name_interned_str\x18\t \x01(\x05H\x00\x12\x11\n\tconsuming\x18\x02 \x01(\x08\x12$\n\x0b\x63ontrollers\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\"\n\tobservers\x18\x0b \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12*\n\narg_binder\x18\x04 \x01(\x0b\x32\x16.daml_lf_1.VarWithType\x12!\n\x08ret_type\x18\x05 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12\x1f\n\x06update\x18\x06 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x19\n\x0fself_binder_str\x18\x07 \x01(\tH\x01\x12\"\n\x18self_binder_interned_str\x18\n \x01(\x05H\x01\x12%\n\x08location\x18\x08 \x01(\x0b\x32\x13.daml_lf_1.LocationB\x06\n\x04nameB\r\n\x0bself_binder\"\xf0\x03\n\x07KeyExpr\x12\x35\n\x0bprojections\x18\x01 \x01(\x0b\x32\x1e.daml_lf_1.KeyExpr.ProjectionsH\x00\x12+\n\x06record\x18\x02 \x01(\x0b\x32\x19.daml_lf_1.KeyExpr.RecordH\x00\x1al\n\nProjection\x12\"\n\x05tycon\x18\x01 \x01(\x0b\x32\x13.daml_lf_1.Type.Con\x12\x13\n\tfield_str\x18\x02 \x01(\tH\x00\x12\x1c\n\x12\x66ield_interned_str\x18\x03 \x01(\x05H\x00\x42\x07\n\x05\x66ield\x1a\x41\n\x0bProjections\x12\x32\n\x0bprojections\x18\x02 \x03(\x0b\x32\x1d.daml_lf_1.KeyExpr.Projection\x1ak\n\x0bRecordField\x12\x13\n\tfield_str\x18\x01 \x01(\tH\x00\x12\x1c\n\x12\x66ield_interned_str\x18\x03 \x01(\x05H\x00\x12 \n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x12.daml_lf_1.KeyExprB\x07\n\x05\x66ield\x1a\\\n\x06Record\x12\"\n\x05tycon\x18\x01 \x01(\x0b\x32\x13.daml_lf_1.Type.Con\x12.\n\x06\x66ields\x18\x02 \x03(\x0b\x32\x1e.daml_lf_1.KeyExpr.RecordFieldB\x05\n\x03Sum\"\xdc\x04\n\x0b\x44\x65\x66Template\x12,\n\x0btycon_dname\x18\x01 \x01(\x0b\x32\x15.daml_lf_1.DottedNameH\x00\x12\x1e\n\x14tycon_interned_dname\x18\x0c \x01(\x05H\x00\x12\x13\n\tparam_str\x18\x02 \x01(\tH\x01\x12\x1c\n\x12param_interned_str\x18\x0b \x01(\x05H\x01\x12 \n\x07precond\x18\x04 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12$\n\x0bsignatories\x18\x05 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\"\n\tagreement\x18\x06 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12*\n\x07\x63hoices\x18\x07 \x03(\x0b\x32\x19.daml_lf_1.TemplateChoice\x12\"\n\tobservers\x18\x08 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12%\n\x08location\x18\t \x01(\x0b\x32\x13.daml_lf_1.Location\x12*\n\x03key\x18\n \x01(\x0b\x32\x1d.daml_lf_1.DefTemplate.DefKey\x1a\xa4\x01\n\x06\x44\x65\x66Key\x12\x1d\n\x04type\x18\x01 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12!\n\x03key\x18\x02 \x01(\x0b\x32\x12.daml_lf_1.KeyExprH\x00\x12&\n\x0b\x63omplex_key\x18\x04 \x01(\x0b\x32\x0f.daml_lf_1.ExprH\x00\x12$\n\x0bmaintainers\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.ExprB\n\n\x08key_exprB\x07\n\x05tyconB\x07\n\x05paramJ\x04\x08\x03\x10\x04\"t\n\x0c\x44\x65\x66\x45xception\x12\x1b\n\x13name_interned_dname\x18\x01 \x01(\x05\x12%\n\x08location\x18\x02 \x01(\x0b\x32\x13.daml_lf_1.Location\x12 \n\x07message\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.Expr\"\xf7\x03\n\x0b\x44\x65\x66\x44\x61taType\x12+\n\nname_dname\x18\x01 \x01(\x0b\x32\x15.daml_lf_1.DottedNameH\x00\x12\x1d\n\x13name_interned_dname\x18\x08 \x01(\x05H\x00\x12*\n\x06params\x18\x02 \x03(\x0b\x32\x1a.daml_lf_1.TypeVarWithKind\x12/\n\x06record\x18\x03 \x01(\x0b\x32\x1d.daml_lf_1.DefDataType.FieldsH\x01\x12\x30\n\x07variant\x18\x04 \x01(\x0b\x32\x1d.daml_lf_1.DefDataType.FieldsH\x01\x12\x37\n\x04\x65num\x18\x07 \x01(\x0b\x32\'.daml_lf_1.DefDataType.EnumConstructorsH\x01\x12\x14\n\x0cserializable\x18\x05 \x01(\x08\x12%\n\x08location\x18\x06 \x01(\x0b\x32\x13.daml_lf_1.Location\x1a\x32\n\x06\x46ields\x12(\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x18.daml_lf_1.FieldWithType\x1aO\n\x10\x45numConstructors\x12\x18\n\x10\x63onstructors_str\x18\x01 \x03(\t\x12!\n\x19\x63onstructors_interned_str\x18\x02 \x03(\x05\x42\x06\n\x04nameB\n\n\x08\x44\x61taCons\"\xd2\x01\n\nDefTypeSyn\x12+\n\nname_dname\x18\x01 \x01(\x0b\x32\x15.daml_lf_1.DottedNameH\x00\x12\x1d\n\x13name_interned_dname\x18\x08 \x01(\x05H\x00\x12*\n\x06params\x18\x02 \x03(\x0b\x32\x1a.daml_lf_1.TypeVarWithKind\x12\x1d\n\x04type\x18\x03 \x01(\x0b\x32\x0f.daml_lf_1.Type\x12%\n\x08location\x18\x04 \x01(\x0b\x32\x13.daml_lf_1.LocationB\x06\n\x04name\"\x96\x02\n\x08\x44\x65\x66Value\x12\x38\n\x0ename_with_type\x18\x01 \x01(\x0b\x32 .daml_lf_1.DefValue.NameWithType\x12\x1d\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Expr\x12\x19\n\x11no_party_literals\x18\x03 \x01(\x08\x12\x0f\n\x07is_test\x18\x04 \x01(\x08\x12%\n\x08location\x18\x05 \x01(\x0b\x32\x13.daml_lf_1.Location\x1a^\n\x0cNameWithType\x12\x12\n\nname_dname\x18\x01 \x03(\t\x12\x1b\n\x13name_interned_dname\x18\x03 \x01(\x05\x12\x1d\n\x04type\x18\x02 \x01(\x0b\x32\x0f.daml_lf_1.Type\"\x90\x01\n\x0c\x46\x65\x61tureFlags\x12\x1b\n\x13\x66orbidPartyLiterals\x18\x01 \x01(\x08\x12/\n\'dontDivulgeContractIdsInCreateArguments\x18\x02 \x01(\x08\x12\x32\n*dontDiscloseNonConsumingChoicesToObservers\x18\x03 \x01(\x08\"\xe2\x02\n\x06Module\x12+\n\nname_dname\x18\x01 \x01(\x0b\x32\x15.daml_lf_1.DottedNameH\x00\x12\x1d\n\x13name_interned_dname\x18\x08 \x01(\x05H\x00\x12&\n\x05\x66lags\x18\x04 \x01(\x0b\x32\x17.daml_lf_1.FeatureFlags\x12\'\n\x08synonyms\x18\t \x03(\x0b\x32\x15.daml_lf_1.DefTypeSyn\x12*\n\ndata_types\x18\x05 \x03(\x0b\x32\x16.daml_lf_1.DefDataType\x12#\n\x06values\x18\x06 \x03(\x0b\x32\x13.daml_lf_1.DefValue\x12)\n\ttemplates\x18\x07 \x03(\x0b\x32\x16.daml_lf_1.DefTemplate\x12+\n\nexceptions\x18\n \x03(\x0b\x32\x17.daml_lf_1.DefExceptionB\x06\n\x04nameJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04\"3\n\x12InternedDottedName\x12\x1d\n\x15segments_interned_str\x18\x01 \x03(\x05\"J\n\x0fPackageMetadata\x12\x19\n\x11name_interned_str\x18\x01 \x01(\x05\x12\x1c\n\x14version_interned_str\x18\x02 \x01(\x05\"\xdc\x01\n\x07Package\x12\"\n\x07modules\x18\x01 \x03(\x0b\x32\x11.daml_lf_1.Module\x12\x18\n\x10interned_strings\x18\x02 \x03(\t\x12<\n\x15interned_dotted_names\x18\x03 \x03(\x0b\x32\x1d.daml_lf_1.InternedDottedName\x12,\n\x08metadata\x18\x04 \x01(\x0b\x32\x1a.daml_lf_1.PackageMetadata\x12\'\n\x0einterned_types\x18\x05 \x03(\x0b\x32\x0f.daml_lf_1.Type*\xbf\x02\n\x08PrimType\x12\x08\n\x04UNIT\x10\x00\x12\x08\n\x04\x42OOL\x10\x01\x12\t\n\x05INT64\x10\x02\x12\x0b\n\x07\x44\x45\x43IMAL\x10\x03\x12\x08\n\x04TEXT\x10\x05\x12\r\n\tTIMESTAMP\x10\x06\x12\t\n\x05PARTY\x10\x08\x12\x08\n\x04LIST\x10\t\x12\n\n\x06UPDATE\x10\n\x12\x0c\n\x08SCENARIO\x10\x0b\x12\x08\n\x04\x44\x41TE\x10\x0c\x12\x0f\n\x0b\x43ONTRACT_ID\x10\r\x12\x0c\n\x08OPTIONAL\x10\x0e\x12\t\n\x05\x41RROW\x10\x0f\x12\x0b\n\x07TEXTMAP\x10\x10\x12\x0b\n\x07NUMERIC\x10\x11\x12\x07\n\x03\x41NY\x10\x12\x12\x0c\n\x08TYPE_REP\x10\x13\x12\n\n\x06GENMAP\x10\x14\x12\x11\n\rANY_EXCEPTION\x10\x15\x12\x11\n\rGENERAL_ERROR\x10\x16\x12\x14\n\x10\x41RITHMETIC_ERROR\x10\x17\x12\x12\n\x0e\x43ONTRACT_ERROR\x10\x18*4\n\x07PrimCon\x12\x0c\n\x08\x43ON_UNIT\x10\x00\x12\r\n\tCON_FALSE\x10\x01\x12\x0c\n\x08\x43ON_TRUE\x10\x02*\xb1\x12\n\x0f\x42uiltinFunction\x12\x0f\n\x0b\x41\x44\x44_DECIMAL\x10\x00\x12\x0f\n\x0bSUB_DECIMAL\x10\x01\x12\x0f\n\x0bMUL_DECIMAL\x10\x02\x12\x0f\n\x0b\x44IV_DECIMAL\x10\x03\x12\x11\n\rROUND_DECIMAL\x10\x06\x12\x0f\n\x0b\x41\x44\x44_NUMERIC\x10k\x12\x0f\n\x0bSUB_NUMERIC\x10l\x12\x0f\n\x0bMUL_NUMERIC\x10m\x12\x0f\n\x0b\x44IV_NUMERIC\x10n\x12\x11\n\rROUND_NUMERIC\x10o\x12\x10\n\x0c\x43\x41ST_NUMERIC\x10y\x12\x11\n\rSHIFT_NUMERIC\x10z\x12\r\n\tADD_INT64\x10\x07\x12\r\n\tSUB_INT64\x10\x08\x12\r\n\tMUL_INT64\x10\t\x12\r\n\tDIV_INT64\x10\n\x12\r\n\tMOD_INT64\x10\x0b\x12\r\n\tEXP_INT64\x10\x0c\x12\t\n\x05\x46OLDL\x10\x14\x12\t\n\x05\x46OLDR\x10\x15\x12\x11\n\rTEXTMAP_EMPTY\x10`\x12\x12\n\x0eTEXTMAP_INSERT\x10\x61\x12\x12\n\x0eTEXTMAP_LOOKUP\x10\x62\x12\x12\n\x0eTEXTMAP_DELETE\x10\x63\x12\x13\n\x0fTEXTMAP_TO_LIST\x10\x64\x12\x10\n\x0cTEXTMAP_SIZE\x10\x65\x12\x10\n\x0cGENMAP_EMPTY\x10|\x12\x11\n\rGENMAP_INSERT\x10}\x12\x11\n\rGENMAP_LOOKUP\x10~\x12\x11\n\rGENMAP_DELETE\x10\x7f\x12\x10\n\x0bGENMAP_KEYS\x10\x80\x01\x12\x12\n\rGENMAP_VALUES\x10\x81\x01\x12\x10\n\x0bGENMAP_SIZE\x10\x82\x01\x12\x10\n\x0c\x45XPLODE_TEXT\x10\x17\x12\x0f\n\x0b\x41PPEND_TEXT\x10\x18\x12\t\n\x05\x45RROR\x10\x19\x12\x1a\n\x15\x41NY_EXCEPTION_MESSAGE\x10\x89\x01\x12\x17\n\x12MAKE_GENERAL_ERROR\x10\x8a\x01\x12\x1a\n\x15GENERAL_ERROR_MESSAGE\x10\x8b\x01\x12\x1a\n\x15MAKE_ARITHMETIC_ERROR\x10\x8c\x01\x12\x1d\n\x18\x41RITHMETIC_ERROR_MESSAGE\x10\x8d\x01\x12\x18\n\x13MAKE_CONTRACT_ERROR\x10\x8e\x01\x12\x1b\n\x16\x43ONTRACT_ERROR_MESSAGE\x10\x8f\x01\x12\r\n\tLEQ_INT64\x10!\x12\x0f\n\x0bLEQ_DECIMAL\x10\"\x12\x0f\n\x0bLEQ_NUMERIC\x10p\x12\x0c\n\x08LEQ_TEXT\x10$\x12\x11\n\rLEQ_TIMESTAMP\x10%\x12\x0c\n\x08LEQ_DATE\x10\x43\x12\r\n\tLEQ_PARTY\x10Y\x12\x0e\n\nLESS_INT64\x10\'\x12\x10\n\x0cLESS_DECIMAL\x10(\x12\x10\n\x0cLESS_NUMERIC\x10q\x12\r\n\tLESS_TEXT\x10*\x12\x12\n\x0eLESS_TIMESTAMP\x10+\x12\r\n\tLESS_DATE\x10\x44\x12\x0e\n\nLESS_PARTY\x10Z\x12\r\n\tGEQ_INT64\x10-\x12\x0f\n\x0bGEQ_DECIMAL\x10.\x12\x0f\n\x0bGEQ_NUMERIC\x10r\x12\x0c\n\x08GEQ_TEXT\x10\x30\x12\x11\n\rGEQ_TIMESTAMP\x10\x31\x12\x0c\n\x08GEQ_DATE\x10\x45\x12\r\n\tGEQ_PARTY\x10[\x12\x11\n\rGREATER_INT64\x10\x33\x12\x13\n\x0fGREATER_DECIMAL\x10\x34\x12\x13\n\x0fGREATER_NUMERIC\x10s\x12\x10\n\x0cGREATER_TEXT\x10\x36\x12\x15\n\x11GREATER_TIMESTAMP\x10\x37\x12\x10\n\x0cGREATER_DATE\x10\x46\x12\x11\n\rGREATER_PARTY\x10\\\x12\x11\n\rTO_TEXT_INT64\x10\x39\x12\x13\n\x0fTO_TEXT_DECIMAL\x10:\x12\x13\n\x0fTO_TEXT_NUMERIC\x10t\x12\x10\n\x0cTO_TEXT_TEXT\x10<\x12\x15\n\x11TO_TEXT_TIMESTAMP\x10=\x12\x10\n\x0cTO_TEXT_DATE\x10G\x12\x18\n\x14TO_QUOTED_TEXT_PARTY\x10?\x12\x11\n\rTO_TEXT_PARTY\x10^\x12\x13\n\x0f\x46ROM_TEXT_PARTY\x10_\x12\x13\n\x0f\x46ROM_TEXT_INT64\x10g\x12\x15\n\x11\x46ROM_TEXT_DECIMAL\x10h\x12\x15\n\x11\x46ROM_TEXT_NUMERIC\x10u\x12\x18\n\x13TO_TEXT_CONTRACT_ID\x10\x88\x01\x12\x0f\n\x0bSHA256_TEXT\x10]\x12\x15\n\x11\x44\x41TE_TO_UNIX_DAYS\x10H\x12\x15\n\x11UNIX_DAYS_TO_DATE\x10I\x12\"\n\x1eTIMESTAMP_TO_UNIX_MICROSECONDS\x10J\x12\"\n\x1eUNIX_MICROSECONDS_TO_TIMESTAMP\x10K\x12\x14\n\x10INT64_TO_DECIMAL\x10L\x12\x14\n\x10\x44\x45\x43IMAL_TO_INT64\x10M\x12\x14\n\x10INT64_TO_NUMERIC\x10v\x12\x14\n\x10NUMERIC_TO_INT64\x10w\x12\x10\n\x0cIMPLODE_TEXT\x10N\x12\x0f\n\x0b\x45QUAL_INT64\x10O\x12\x11\n\rEQUAL_DECIMAL\x10P\x12\x11\n\rEQUAL_NUMERIC\x10x\x12\x0e\n\nEQUAL_TEXT\x10Q\x12\x13\n\x0f\x45QUAL_TIMESTAMP\x10R\x12\x0e\n\nEQUAL_DATE\x10S\x12\x0f\n\x0b\x45QUAL_PARTY\x10T\x12\x0e\n\nEQUAL_BOOL\x10U\x12\x15\n\x11\x45QUAL_CONTRACT_ID\x10V\x12\x0e\n\nEQUAL_LIST\x10W\x12\x12\n\x0e\x45QUAL_TYPE_REP\x10{\x12\n\n\x05\x45QUAL\x10\x83\x01\x12\x0c\n\x07LESS_EQ\x10\x84\x01\x12\t\n\x04LESS\x10\x85\x01\x12\x0f\n\nGREATER_EQ\x10\x86\x01\x12\x0c\n\x07GREATER\x10\x87\x01\x12\t\n\x05TRACE\x10X\x12\x16\n\x12\x43OERCE_CONTRACT_ID\x10\x66\x12\x19\n\x15TEXT_FROM_CODE_POINTS\x10i\x12\x17\n\x13TEXT_TO_CODE_POINTS\x10j\x12\x12\n\rTEXT_TO_UPPER\x10\xadM\x12\x12\n\rTEXT_TO_LOWER\x10\xaeM\x12\x0f\n\nTEXT_SLICE\x10\xafM\x12\x15\n\x10TEXT_SLICE_INDEX\x10\xb0M\x12\x17\n\x12TEXT_CONTAINS_ONLY\x10\xb1M\x12\x13\n\x0eTEXT_REPLICATE\x10\xb2M\x12\x12\n\rTEXT_SPLIT_ON\x10\xb3M\x12\x15\n\x10TEXT_INTERCALATE\x10\xb4MB5\n\x14\x63om.daml.daml_lf_dev\xaa\x02\x1c\x43om.Daml.Daml_Lf_Dev.DamlLf1b\x06proto3'
)

_PRIMTYPE = _descriptor.EnumDescriptor(
  name='PrimType',
  full_name='daml_lf_1.PrimType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNIT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BOOL', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INT64', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DECIMAL', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT', index=4, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TIMESTAMP', index=5, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARTY', index=6, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LIST', index=7, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UPDATE', index=8, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SCENARIO', index=9, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATE', index=10, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONTRACT_ID', index=11, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OPTIONAL', index=12, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ARROW', index=13, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXTMAP', index=14, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NUMERIC', index=15, number=17,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ANY', index=16, number=18,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TYPE_REP', index=17, number=19,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENMAP', index=18, number=20,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ANY_EXCEPTION', index=19, number=21,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENERAL_ERROR', index=20, number=22,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ARITHMETIC_ERROR', index=21, number=23,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONTRACT_ERROR', index=22, number=24,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12605,
  serialized_end=12924,
)
_sym_db.RegisterEnumDescriptor(_PRIMTYPE)

PrimType = enum_type_wrapper.EnumTypeWrapper(_PRIMTYPE)
_PRIMCON = _descriptor.EnumDescriptor(
  name='PrimCon',
  full_name='daml_lf_1.PrimCon',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CON_UNIT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CON_FALSE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CON_TRUE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12926,
  serialized_end=12978,
)
_sym_db.RegisterEnumDescriptor(_PRIMCON)

PrimCon = enum_type_wrapper.EnumTypeWrapper(_PRIMCON)
_BUILTINFUNCTION = _descriptor.EnumDescriptor(
  name='BuiltinFunction',
  full_name='daml_lf_1.BuiltinFunction',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ADD_DECIMAL', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SUB_DECIMAL', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MUL_DECIMAL', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DIV_DECIMAL', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ROUND_DECIMAL', index=4, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ADD_NUMERIC', index=5, number=107,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SUB_NUMERIC', index=6, number=108,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MUL_NUMERIC', index=7, number=109,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DIV_NUMERIC', index=8, number=110,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ROUND_NUMERIC', index=9, number=111,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CAST_NUMERIC', index=10, number=121,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SHIFT_NUMERIC', index=11, number=122,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ADD_INT64', index=12, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SUB_INT64', index=13, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MUL_INT64', index=14, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DIV_INT64', index=15, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MOD_INT64', index=16, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXP_INT64', index=17, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FOLDL', index=18, number=20,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FOLDR', index=19, number=21,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXTMAP_EMPTY', index=20, number=96,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXTMAP_INSERT', index=21, number=97,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXTMAP_LOOKUP', index=22, number=98,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXTMAP_DELETE', index=23, number=99,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXTMAP_TO_LIST', index=24, number=100,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXTMAP_SIZE', index=25, number=101,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENMAP_EMPTY', index=26, number=124,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENMAP_INSERT', index=27, number=125,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENMAP_LOOKUP', index=28, number=126,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENMAP_DELETE', index=29, number=127,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENMAP_KEYS', index=30, number=128,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENMAP_VALUES', index=31, number=129,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENMAP_SIZE', index=32, number=130,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXPLODE_TEXT', index=33, number=23,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='APPEND_TEXT', index=34, number=24,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=35, number=25,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ANY_EXCEPTION_MESSAGE', index=36, number=137,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MAKE_GENERAL_ERROR', index=37, number=138,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENERAL_ERROR_MESSAGE', index=38, number=139,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MAKE_ARITHMETIC_ERROR', index=39, number=140,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ARITHMETIC_ERROR_MESSAGE', index=40, number=141,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MAKE_CONTRACT_ERROR', index=41, number=142,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONTRACT_ERROR_MESSAGE', index=42, number=143,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEQ_INT64', index=43, number=33,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEQ_DECIMAL', index=44, number=34,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEQ_NUMERIC', index=45, number=112,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEQ_TEXT', index=46, number=36,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEQ_TIMESTAMP', index=47, number=37,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEQ_DATE', index=48, number=67,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEQ_PARTY', index=49, number=89,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS_INT64', index=50, number=39,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS_DECIMAL', index=51, number=40,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS_NUMERIC', index=52, number=113,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS_TEXT', index=53, number=42,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS_TIMESTAMP', index=54, number=43,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS_DATE', index=55, number=68,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS_PARTY', index=56, number=90,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GEQ_INT64', index=57, number=45,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GEQ_DECIMAL', index=58, number=46,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GEQ_NUMERIC', index=59, number=114,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GEQ_TEXT', index=60, number=48,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GEQ_TIMESTAMP', index=61, number=49,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GEQ_DATE', index=62, number=69,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GEQ_PARTY', index=63, number=91,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_INT64', index=64, number=51,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_DECIMAL', index=65, number=52,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_NUMERIC', index=66, number=115,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_TEXT', index=67, number=54,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_TIMESTAMP', index=68, number=55,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_DATE', index=69, number=70,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_PARTY', index=70, number=92,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TO_TEXT_INT64', index=71, number=57,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TO_TEXT_DECIMAL', index=72, number=58,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TO_TEXT_NUMERIC', index=73, number=116,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TO_TEXT_TEXT', index=74, number=60,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TO_TEXT_TIMESTAMP', index=75, number=61,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TO_TEXT_DATE', index=76, number=71,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TO_QUOTED_TEXT_PARTY', index=77, number=63,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TO_TEXT_PARTY', index=78, number=94,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FROM_TEXT_PARTY', index=79, number=95,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FROM_TEXT_INT64', index=80, number=103,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FROM_TEXT_DECIMAL', index=81, number=104,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FROM_TEXT_NUMERIC', index=82, number=117,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TO_TEXT_CONTRACT_ID', index=83, number=136,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SHA256_TEXT', index=84, number=93,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATE_TO_UNIX_DAYS', index=85, number=72,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNIX_DAYS_TO_DATE', index=86, number=73,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TIMESTAMP_TO_UNIX_MICROSECONDS', index=87, number=74,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNIX_MICROSECONDS_TO_TIMESTAMP', index=88, number=75,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INT64_TO_DECIMAL', index=89, number=76,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DECIMAL_TO_INT64', index=90, number=77,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INT64_TO_NUMERIC', index=91, number=118,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NUMERIC_TO_INT64', index=92, number=119,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='IMPLODE_TEXT', index=93, number=78,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_INT64', index=94, number=79,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_DECIMAL', index=95, number=80,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_NUMERIC', index=96, number=120,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_TEXT', index=97, number=81,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_TIMESTAMP', index=98, number=82,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_DATE', index=99, number=83,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_PARTY', index=100, number=84,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_BOOL', index=101, number=85,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_CONTRACT_ID', index=102, number=86,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_LIST', index=103, number=87,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_TYPE_REP', index=104, number=123,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL', index=105, number=131,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS_EQ', index=106, number=132,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS', index=107, number=133,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_EQ', index=108, number=134,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER', index=109, number=135,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRACE', index=110, number=88,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COERCE_CONTRACT_ID', index=111, number=102,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_FROM_CODE_POINTS', index=112, number=105,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_TO_CODE_POINTS', index=113, number=106,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_TO_UPPER', index=114, number=9901,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_TO_LOWER', index=115, number=9902,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_SLICE', index=116, number=9903,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_SLICE_INDEX', index=117, number=9904,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_CONTAINS_ONLY', index=118, number=9905,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_REPLICATE', index=119, number=9906,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_SPLIT_ON', index=120, number=9907,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT_INTERCALATE', index=121, number=9908,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12981,
  serialized_end=15334,
)
_sym_db.RegisterEnumDescriptor(_BUILTINFUNCTION)

BuiltinFunction = enum_type_wrapper.EnumTypeWrapper(_BUILTINFUNCTION)
UNIT = 0
BOOL = 1
INT64 = 2
DECIMAL = 3
TEXT = 5
TIMESTAMP = 6
PARTY = 8
LIST = 9
UPDATE = 10
SCENARIO = 11
DATE = 12
CONTRACT_ID = 13
OPTIONAL = 14
ARROW = 15
TEXTMAP = 16
NUMERIC = 17
ANY = 18
TYPE_REP = 19
GENMAP = 20
ANY_EXCEPTION = 21
GENERAL_ERROR = 22
ARITHMETIC_ERROR = 23
CONTRACT_ERROR = 24
CON_UNIT = 0
CON_FALSE = 1
CON_TRUE = 2
ADD_DECIMAL = 0
SUB_DECIMAL = 1
MUL_DECIMAL = 2
DIV_DECIMAL = 3
ROUND_DECIMAL = 6
ADD_NUMERIC = 107
SUB_NUMERIC = 108
MUL_NUMERIC = 109
DIV_NUMERIC = 110
ROUND_NUMERIC = 111
CAST_NUMERIC = 121
SHIFT_NUMERIC = 122
ADD_INT64 = 7
SUB_INT64 = 8
MUL_INT64 = 9
DIV_INT64 = 10
MOD_INT64 = 11
EXP_INT64 = 12
FOLDL = 20
FOLDR = 21
TEXTMAP_EMPTY = 96
TEXTMAP_INSERT = 97
TEXTMAP_LOOKUP = 98
TEXTMAP_DELETE = 99
TEXTMAP_TO_LIST = 100
TEXTMAP_SIZE = 101
GENMAP_EMPTY = 124
GENMAP_INSERT = 125
GENMAP_LOOKUP = 126
GENMAP_DELETE = 127
GENMAP_KEYS = 128
GENMAP_VALUES = 129
GENMAP_SIZE = 130
EXPLODE_TEXT = 23
APPEND_TEXT = 24
ERROR = 25
ANY_EXCEPTION_MESSAGE = 137
MAKE_GENERAL_ERROR = 138
GENERAL_ERROR_MESSAGE = 139
MAKE_ARITHMETIC_ERROR = 140
ARITHMETIC_ERROR_MESSAGE = 141
MAKE_CONTRACT_ERROR = 142
CONTRACT_ERROR_MESSAGE = 143
LEQ_INT64 = 33
LEQ_DECIMAL = 34
LEQ_NUMERIC = 112
LEQ_TEXT = 36
LEQ_TIMESTAMP = 37
LEQ_DATE = 67
LEQ_PARTY = 89
LESS_INT64 = 39
LESS_DECIMAL = 40
LESS_NUMERIC = 113
LESS_TEXT = 42
LESS_TIMESTAMP = 43
LESS_DATE = 68
LESS_PARTY = 90
GEQ_INT64 = 45
GEQ_DECIMAL = 46
GEQ_NUMERIC = 114
GEQ_TEXT = 48
GEQ_TIMESTAMP = 49
GEQ_DATE = 69
GEQ_PARTY = 91
GREATER_INT64 = 51
GREATER_DECIMAL = 52
GREATER_NUMERIC = 115
GREATER_TEXT = 54
GREATER_TIMESTAMP = 55
GREATER_DATE = 70
GREATER_PARTY = 92
TO_TEXT_INT64 = 57
TO_TEXT_DECIMAL = 58
TO_TEXT_NUMERIC = 116
TO_TEXT_TEXT = 60
TO_TEXT_TIMESTAMP = 61
TO_TEXT_DATE = 71
TO_QUOTED_TEXT_PARTY = 63
TO_TEXT_PARTY = 94
FROM_TEXT_PARTY = 95
FROM_TEXT_INT64 = 103
FROM_TEXT_DECIMAL = 104
FROM_TEXT_NUMERIC = 117
TO_TEXT_CONTRACT_ID = 136
SHA256_TEXT = 93
DATE_TO_UNIX_DAYS = 72
UNIX_DAYS_TO_DATE = 73
TIMESTAMP_TO_UNIX_MICROSECONDS = 74
UNIX_MICROSECONDS_TO_TIMESTAMP = 75
INT64_TO_DECIMAL = 76
DECIMAL_TO_INT64 = 77
INT64_TO_NUMERIC = 118
NUMERIC_TO_INT64 = 119
IMPLODE_TEXT = 78
EQUAL_INT64 = 79
EQUAL_DECIMAL = 80
EQUAL_NUMERIC = 120
EQUAL_TEXT = 81
EQUAL_TIMESTAMP = 82
EQUAL_DATE = 83
EQUAL_PARTY = 84
EQUAL_BOOL = 85
EQUAL_CONTRACT_ID = 86
EQUAL_LIST = 87
EQUAL_TYPE_REP = 123
EQUAL = 131
LESS_EQ = 132
LESS = 133
GREATER_EQ = 134
GREATER = 135
TRACE = 88
COERCE_CONTRACT_ID = 102
TEXT_FROM_CODE_POINTS = 105
TEXT_TO_CODE_POINTS = 106
TEXT_TO_UPPER = 9901
TEXT_TO_LOWER = 9902
TEXT_SLICE = 9903
TEXT_SLICE_INDEX = 9904
TEXT_CONTAINS_ONLY = 9905
TEXT_REPLICATE = 9906
TEXT_SPLIT_ON = 9907
TEXT_INTERCALATE = 9908



_UNIT = _descriptor.Descriptor(
  name='Unit',
  full_name='daml_lf_1.Unit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=51,
  serialized_end=57,
)


_PACKAGEREF = _descriptor.Descriptor(
  name='PackageRef',
  full_name='daml_lf_1.PackageRef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='self', full_name='daml_lf_1.PackageRef.self', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='package_id_str', full_name='daml_lf_1.PackageRef.package_id_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='package_id_interned_str', full_name='daml_lf_1.PackageRef.package_id_interned_str', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='daml_lf_1.PackageRef.Sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=59,
  serialized_end=172,
)


_DOTTEDNAME = _descriptor.Descriptor(
  name='DottedName',
  full_name='daml_lf_1.DottedName',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='segments', full_name='daml_lf_1.DottedName.segments', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=174,
  serialized_end=204,
)


_MODULEREF = _descriptor.Descriptor(
  name='ModuleRef',
  full_name='daml_lf_1.ModuleRef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='package_ref', full_name='daml_lf_1.ModuleRef.package_ref', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='module_name_dname', full_name='daml_lf_1.ModuleRef.module_name_dname', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='module_name_interned_dname', full_name='daml_lf_1.ModuleRef.module_name_interned_dname', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='module_name', full_name='daml_lf_1.ModuleRef.module_name',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=207,
  serialized_end=367,
)


_TYPECONNAME = _descriptor.Descriptor(
  name='TypeConName',
  full_name='daml_lf_1.TypeConName',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='module', full_name='daml_lf_1.TypeConName.module', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_dname', full_name='daml_lf_1.TypeConName.name_dname', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_interned_dname', full_name='daml_lf_1.TypeConName.name_interned_dname', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='name', full_name='daml_lf_1.TypeConName.name',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=370,
  serialized_end=505,
)


_TYPESYNNAME = _descriptor.Descriptor(
  name='TypeSynName',
  full_name='daml_lf_1.TypeSynName',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='module', full_name='daml_lf_1.TypeSynName.module', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_dname', full_name='daml_lf_1.TypeSynName.name_dname', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_interned_dname', full_name='daml_lf_1.TypeSynName.name_interned_dname', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='name', full_name='daml_lf_1.TypeSynName.name',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=508,
  serialized_end=643,
)


_VALNAME = _descriptor.Descriptor(
  name='ValName',
  full_name='daml_lf_1.ValName',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='module', full_name='daml_lf_1.ValName.module', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_dname', full_name='daml_lf_1.ValName.name_dname', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_interned_dname', full_name='daml_lf_1.ValName.name_interned_dname', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=645,
  serialized_end=741,
)


_FIELDWITHTYPE = _descriptor.Descriptor(
  name='FieldWithType',
  full_name='daml_lf_1.FieldWithType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='field_str', full_name='daml_lf_1.FieldWithType.field_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_interned_str', full_name='daml_lf_1.FieldWithType.field_interned_str', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.FieldWithType.type', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field', full_name='daml_lf_1.FieldWithType.field',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=743,
  serialized_end=849,
)


_VARWITHTYPE = _descriptor.Descriptor(
  name='VarWithType',
  full_name='daml_lf_1.VarWithType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='var_str', full_name='daml_lf_1.VarWithType.var_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_interned_str', full_name='daml_lf_1.VarWithType.var_interned_str', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.VarWithType.type', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='var', full_name='daml_lf_1.VarWithType.var',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=851,
  serialized_end=949,
)


_TYPEVARWITHKIND = _descriptor.Descriptor(
  name='TypeVarWithKind',
  full_name='daml_lf_1.TypeVarWithKind',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='var_str', full_name='daml_lf_1.TypeVarWithKind.var_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_interned_str', full_name='daml_lf_1.TypeVarWithKind.var_interned_str', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kind', full_name='daml_lf_1.TypeVarWithKind.kind', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='var', full_name='daml_lf_1.TypeVarWithKind.var',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=951,
  serialized_end=1053,
)


_FIELDWITHEXPR = _descriptor.Descriptor(
  name='FieldWithExpr',
  full_name='daml_lf_1.FieldWithExpr',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='field_str', full_name='daml_lf_1.FieldWithExpr.field_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_interned_str', full_name='daml_lf_1.FieldWithExpr.field_interned_str', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.FieldWithExpr.expr', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field', full_name='daml_lf_1.FieldWithExpr.field',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1055,
  serialized_end=1161,
)


_BINDING = _descriptor.Descriptor(
  name='Binding',
  full_name='daml_lf_1.Binding',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='binder', full_name='daml_lf_1.Binding.binder', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bound', full_name='daml_lf_1.Binding.bound', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1163,
  serialized_end=1244,
)


_KIND_ARROW = _descriptor.Descriptor(
  name='Arrow',
  full_name='daml_lf_1.Kind.Arrow',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='params', full_name='daml_lf_1.Kind.Arrow.params', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='daml_lf_1.Kind.Arrow.result', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1360,
  serialized_end=1433,
)

_KIND = _descriptor.Descriptor(
  name='Kind',
  full_name='daml_lf_1.Kind',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='star', full_name='daml_lf_1.Kind.star', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arrow', full_name='daml_lf_1.Kind.arrow', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nat', full_name='daml_lf_1.Kind.nat', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_KIND_ARROW, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='daml_lf_1.Kind.Sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1247,
  serialized_end=1440,
)


_TYPE_VAR = _descriptor.Descriptor(
  name='Var',
  full_name='daml_lf_1.Type.Var',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='var_str', full_name='daml_lf_1.Type.Var.var_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_interned_str', full_name='daml_lf_1.Type.Var.var_interned_str', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='daml_lf_1.Type.Var.args', index=2,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='var', full_name='daml_lf_1.Type.Var.var',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1716,
  serialized_end=1806,
)

_TYPE_CON = _descriptor.Descriptor(
  name='Con',
  full_name='daml_lf_1.Type.Con',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tycon', full_name='daml_lf_1.Type.Con.tycon', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='daml_lf_1.Type.Con.args', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1808,
  serialized_end=1883,
)

_TYPE_SYN = _descriptor.Descriptor(
  name='Syn',
  full_name='daml_lf_1.Type.Syn',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tysyn', full_name='daml_lf_1.Type.Syn.tysyn', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='daml_lf_1.Type.Syn.args', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1885,
  serialized_end=1960,
)

_TYPE_PRIM = _descriptor.Descriptor(
  name='Prim',
  full_name='daml_lf_1.Type.Prim',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='prim', full_name='daml_lf_1.Type.Prim.prim', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='daml_lf_1.Type.Prim.args', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1962,
  serialized_end=2034,
)

_TYPE_FUN = _descriptor.Descriptor(
  name='Fun',
  full_name='daml_lf_1.Type.Fun',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='params', full_name='daml_lf_1.Type.Fun.params', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='daml_lf_1.Type.Fun.result', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2036,
  serialized_end=2107,
)

_TYPE_FORALL = _descriptor.Descriptor(
  name='Forall',
  full_name='daml_lf_1.Type.Forall',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='vars', full_name='daml_lf_1.Type.Forall.vars', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='daml_lf_1.Type.Forall.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2109,
  serialized_end=2190,
)

_TYPE_STRUCT = _descriptor.Descriptor(
  name='Struct',
  full_name='daml_lf_1.Type.Struct',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fields', full_name='daml_lf_1.Type.Struct.fields', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2192,
  serialized_end=2242,
)

_TYPE = _descriptor.Descriptor(
  name='Type',
  full_name='daml_lf_1.Type',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='var', full_name='daml_lf_1.Type.var', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='con', full_name='daml_lf_1.Type.con', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prim', full_name='daml_lf_1.Type.prim', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='forall', full_name='daml_lf_1.Type.forall', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='struct', full_name='daml_lf_1.Type.struct', index=4,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nat', full_name='daml_lf_1.Type.nat', index=5,
      number=11, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='syn', full_name='daml_lf_1.Type.syn', index=6,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='interned', full_name='daml_lf_1.Type.interned', index=7,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_TYPE_VAR, _TYPE_CON, _TYPE_SYN, _TYPE_PRIM, _TYPE_FUN, _TYPE_FORALL, _TYPE_STRUCT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='daml_lf_1.Type.Sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1443,
  serialized_end=2279,
)


_PRIMLIT = _descriptor.Descriptor(
  name='PrimLit',
  full_name='daml_lf_1.PrimLit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='int64', full_name='daml_lf_1.PrimLit.int64', index=0,
      number=1, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='decimal_str', full_name='daml_lf_1.PrimLit.decimal_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='numeric_interned_str', full_name='daml_lf_1.PrimLit.numeric_interned_str', index=2,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='text_str', full_name='daml_lf_1.PrimLit.text_str', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='text_interned_str', full_name='daml_lf_1.PrimLit.text_interned_str', index=4,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='daml_lf_1.PrimLit.timestamp', index=5,
      number=5, type=16, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='party_str', full_name='daml_lf_1.PrimLit.party_str', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='party_interned_str', full_name='daml_lf_1.PrimLit.party_interned_str', index=7,
      number=12, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='date', full_name='daml_lf_1.PrimLit.date', index=8,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='daml_lf_1.PrimLit.Sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2282,
  serialized_end=2519,
)


_LOCATION_RANGE = _descriptor.Descriptor(
  name='Range',
  full_name='daml_lf_1.Location.Range',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_line', full_name='daml_lf_1.Location.Range.start_line', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_col', full_name='daml_lf_1.Location.Range.start_col', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_line', full_name='daml_lf_1.Location.Range.end_line', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_col', full_name='daml_lf_1.Location.Range.end_col', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2614,
  serialized_end=2695,
)

_LOCATION = _descriptor.Descriptor(
  name='Location',
  full_name='daml_lf_1.Location',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='module', full_name='daml_lf_1.Location.module', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='range', full_name='daml_lf_1.Location.range', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LOCATION_RANGE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2522,
  serialized_end=2695,
)


_EXPR_RECCON = _descriptor.Descriptor(
  name='RecCon',
  full_name='daml_lf_1.Expr.RecCon',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tycon', full_name='daml_lf_1.Expr.RecCon.tycon', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fields', full_name='daml_lf_1.Expr.RecCon.fields', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4099,
  serialized_end=4185,
)

_EXPR_RECPROJ = _descriptor.Descriptor(
  name='RecProj',
  full_name='daml_lf_1.Expr.RecProj',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tycon', full_name='daml_lf_1.Expr.RecProj.tycon', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_str', full_name='daml_lf_1.Expr.RecProj.field_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_interned_str', full_name='daml_lf_1.Expr.RecProj.field_interned_str', index=2,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='record', full_name='daml_lf_1.Expr.RecProj.record', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field', full_name='daml_lf_1.Expr.RecProj.field',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=4188,
  serialized_end=4326,
)

_EXPR_RECUPD = _descriptor.Descriptor(
  name='RecUpd',
  full_name='daml_lf_1.Expr.RecUpd',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tycon', full_name='daml_lf_1.Expr.RecUpd.tycon', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_str', full_name='daml_lf_1.Expr.RecUpd.field_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_interned_str', full_name='daml_lf_1.Expr.RecUpd.field_interned_str', index=2,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='record', full_name='daml_lf_1.Expr.RecUpd.record', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='update', full_name='daml_lf_1.Expr.RecUpd.update', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field', full_name='daml_lf_1.Expr.RecUpd.field',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=4329,
  serialized_end=4499,
)

_EXPR_VARIANTCON = _descriptor.Descriptor(
  name='VariantCon',
  full_name='daml_lf_1.Expr.VariantCon',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tycon', full_name='daml_lf_1.Expr.VariantCon.tycon', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variant_con_str', full_name='daml_lf_1.Expr.VariantCon.variant_con_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variant_con_interned_str', full_name='daml_lf_1.Expr.VariantCon.variant_con_interned_str', index=2,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variant_arg', full_name='daml_lf_1.Expr.VariantCon.variant_arg', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='variant_con', full_name='daml_lf_1.Expr.VariantCon.variant_con',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=4502,
  serialized_end=4666,
)

_EXPR_ENUMCON = _descriptor.Descriptor(
  name='EnumCon',
  full_name='daml_lf_1.Expr.EnumCon',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tycon', full_name='daml_lf_1.Expr.EnumCon.tycon', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enum_con_str', full_name='daml_lf_1.Expr.EnumCon.enum_con_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enum_con_interned_str', full_name='daml_lf_1.Expr.EnumCon.enum_con_interned_str', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='enum_con', full_name='daml_lf_1.Expr.EnumCon.enum_con',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=4668,
  serialized_end=4785,
)

_EXPR_STRUCTCON = _descriptor.Descriptor(
  name='StructCon',
  full_name='daml_lf_1.Expr.StructCon',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fields', full_name='daml_lf_1.Expr.StructCon.fields', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4787,
  serialized_end=4840,
)

_EXPR_STRUCTPROJ = _descriptor.Descriptor(
  name='StructProj',
  full_name='daml_lf_1.Expr.StructProj',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='field_str', full_name='daml_lf_1.Expr.StructProj.field_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_interned_str', full_name='daml_lf_1.Expr.StructProj.field_interned_str', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='struct', full_name='daml_lf_1.Expr.StructProj.struct', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field', full_name='daml_lf_1.Expr.StructProj.field',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=4842,
  serialized_end=4947,
)

_EXPR_STRUCTUPD = _descriptor.Descriptor(
  name='StructUpd',
  full_name='daml_lf_1.Expr.StructUpd',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='field_str', full_name='daml_lf_1.Expr.StructUpd.field_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_interned_str', full_name='daml_lf_1.Expr.StructUpd.field_interned_str', index=1,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='struct', full_name='daml_lf_1.Expr.StructUpd.struct', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='update', full_name='daml_lf_1.Expr.StructUpd.update', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field', full_name='daml_lf_1.Expr.StructUpd.field',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=4950,
  serialized_end=5087,
)

_EXPR_APP = _descriptor.Descriptor(
  name='App',
  full_name='daml_lf_1.Expr.App',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fun', full_name='daml_lf_1.Expr.App.fun', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='daml_lf_1.Expr.App.args', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5089,
  serialized_end=5155,
)

_EXPR_TYAPP = _descriptor.Descriptor(
  name='TyApp',
  full_name='daml_lf_1.Expr.TyApp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.Expr.TyApp.expr', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='types', full_name='daml_lf_1.Expr.TyApp.types', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5157,
  serialized_end=5227,
)

_EXPR_ABS = _descriptor.Descriptor(
  name='Abs',
  full_name='daml_lf_1.Expr.Abs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='param', full_name='daml_lf_1.Expr.Abs.param', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='daml_lf_1.Expr.Abs.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5229,
  serialized_end=5304,
)

_EXPR_TYABS = _descriptor.Descriptor(
  name='TyAbs',
  full_name='daml_lf_1.Expr.TyAbs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='param', full_name='daml_lf_1.Expr.TyAbs.param', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='daml_lf_1.Expr.TyAbs.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5306,
  serialized_end=5387,
)

_EXPR_NIL = _descriptor.Descriptor(
  name='Nil',
  full_name='daml_lf_1.Expr.Nil',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Expr.Nil.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5389,
  serialized_end=5425,
)

_EXPR_CONS = _descriptor.Descriptor(
  name='Cons',
  full_name='daml_lf_1.Expr.Cons',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Expr.Cons.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='front', full_name='daml_lf_1.Expr.Cons.front', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tail', full_name='daml_lf_1.Expr.Cons.tail', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5427,
  serialized_end=5527,
)

_EXPR_OPTIONALNONE = _descriptor.Descriptor(
  name='OptionalNone',
  full_name='daml_lf_1.Expr.OptionalNone',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Expr.OptionalNone.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5529,
  serialized_end=5574,
)

_EXPR_OPTIONALSOME = _descriptor.Descriptor(
  name='OptionalSome',
  full_name='daml_lf_1.Expr.OptionalSome',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Expr.OptionalSome.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='daml_lf_1.Expr.OptionalSome.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5576,
  serialized_end=5652,
)

_EXPR_TOANY = _descriptor.Descriptor(
  name='ToAny',
  full_name='daml_lf_1.Expr.ToAny',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Expr.ToAny.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.Expr.ToAny.expr', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5654,
  serialized_end=5723,
)

_EXPR_FROMANY = _descriptor.Descriptor(
  name='FromAny',
  full_name='daml_lf_1.Expr.FromAny',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Expr.FromAny.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.Expr.FromAny.expr', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5725,
  serialized_end=5796,
)

_EXPR_TOANYEXCEPTION = _descriptor.Descriptor(
  name='ToAnyException',
  full_name='daml_lf_1.Expr.ToAnyException',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Expr.ToAnyException.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.Expr.ToAnyException.expr', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5798,
  serialized_end=5876,
)

_EXPR_FROMANYEXCEPTION = _descriptor.Descriptor(
  name='FromAnyException',
  full_name='daml_lf_1.Expr.FromAnyException',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Expr.FromAnyException.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.Expr.FromAnyException.expr', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5878,
  serialized_end=5958,
)

_EXPR_THROW = _descriptor.Descriptor(
  name='Throw',
  full_name='daml_lf_1.Expr.Throw',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_type', full_name='daml_lf_1.Expr.Throw.return_type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exception_type', full_name='daml_lf_1.Expr.Throw.exception_type', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exception_expr', full_name='daml_lf_1.Expr.Throw.exception_expr', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5960,
  serialized_end=6087,
)

_EXPR = _descriptor.Descriptor(
  name='Expr',
  full_name='daml_lf_1.Expr',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='location', full_name='daml_lf_1.Expr.location', index=0,
      number=25, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_str', full_name='daml_lf_1.Expr.var_str', index=1,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_interned_str', full_name='daml_lf_1.Expr.var_interned_str', index=2,
      number=29, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='val', full_name='daml_lf_1.Expr.val', index=3,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='builtin', full_name='daml_lf_1.Expr.builtin', index=4,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prim_con', full_name='daml_lf_1.Expr.prim_con', index=5,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prim_lit', full_name='daml_lf_1.Expr.prim_lit', index=6,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rec_con', full_name='daml_lf_1.Expr.rec_con', index=7,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rec_proj', full_name='daml_lf_1.Expr.rec_proj', index=8,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rec_upd', full_name='daml_lf_1.Expr.rec_upd', index=9,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variant_con', full_name='daml_lf_1.Expr.variant_con', index=10,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enum_con', full_name='daml_lf_1.Expr.enum_con', index=11,
      number=28, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='struct_con', full_name='daml_lf_1.Expr.struct_con', index=12,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='struct_proj', full_name='daml_lf_1.Expr.struct_proj', index=13,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='struct_upd', full_name='daml_lf_1.Expr.struct_upd', index=14,
      number=23, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='app', full_name='daml_lf_1.Expr.app', index=15,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ty_app', full_name='daml_lf_1.Expr.ty_app', index=16,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='abs', full_name='daml_lf_1.Expr.abs', index=17,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ty_abs', full_name='daml_lf_1.Expr.ty_abs', index=18,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='case', full_name='daml_lf_1.Expr.case', index=19,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='let', full_name='daml_lf_1.Expr.let', index=20,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nil', full_name='daml_lf_1.Expr.nil', index=21,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cons', full_name='daml_lf_1.Expr.cons', index=22,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='update', full_name='daml_lf_1.Expr.update', index=23,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scenario', full_name='daml_lf_1.Expr.scenario', index=24,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='optional_none', full_name='daml_lf_1.Expr.optional_none', index=25,
      number=26, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='optional_some', full_name='daml_lf_1.Expr.optional_some', index=26,
      number=27, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='to_any', full_name='daml_lf_1.Expr.to_any', index=27,
      number=30, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='from_any', full_name='daml_lf_1.Expr.from_any', index=28,
      number=31, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type_rep', full_name='daml_lf_1.Expr.type_rep', index=29,
      number=32, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='to_any_exception', full_name='daml_lf_1.Expr.to_any_exception', index=30,
      number=33, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='from_any_exception', full_name='daml_lf_1.Expr.from_any_exception', index=31,
      number=34, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='throw', full_name='daml_lf_1.Expr.throw', index=32,
      number=35, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_EXPR_RECCON, _EXPR_RECPROJ, _EXPR_RECUPD, _EXPR_VARIANTCON, _EXPR_ENUMCON, _EXPR_STRUCTCON, _EXPR_STRUCTPROJ, _EXPR_STRUCTUPD, _EXPR_APP, _EXPR_TYAPP, _EXPR_ABS, _EXPR_TYABS, _EXPR_NIL, _EXPR_CONS, _EXPR_OPTIONALNONE, _EXPR_OPTIONALSOME, _EXPR_TOANY, _EXPR_FROMANY, _EXPR_TOANYEXCEPTION, _EXPR_FROMANYEXCEPTION, _EXPR_THROW, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='daml_lf_1.Expr.Sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2698,
  serialized_end=6106,
)


_CASEALT_VARIANT = _descriptor.Descriptor(
  name='Variant',
  full_name='daml_lf_1.CaseAlt.Variant',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='con', full_name='daml_lf_1.CaseAlt.Variant.con', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variant_str', full_name='daml_lf_1.CaseAlt.Variant.variant_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variant_interned_str', full_name='daml_lf_1.CaseAlt.Variant.variant_interned_str', index=2,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='binder_str', full_name='daml_lf_1.CaseAlt.Variant.binder_str', index=3,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='binder_interned_str', full_name='daml_lf_1.CaseAlt.Variant.binder_interned_str', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='variant', full_name='daml_lf_1.CaseAlt.Variant.variant',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='binder', full_name='daml_lf_1.CaseAlt.Variant.binder',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=6489,
  serialized_end=6664,
)

_CASEALT_ENUM = _descriptor.Descriptor(
  name='Enum',
  full_name='daml_lf_1.CaseAlt.Enum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='con', full_name='daml_lf_1.CaseAlt.Enum.con', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='constructor_str', full_name='daml_lf_1.CaseAlt.Enum.constructor_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='constructor_interned_str', full_name='daml_lf_1.CaseAlt.Enum.constructor_interned_str', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='constructor', full_name='daml_lf_1.CaseAlt.Enum.constructor',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=6666,
  serialized_end=6787,
)

_CASEALT_CONS = _descriptor.Descriptor(
  name='Cons',
  full_name='daml_lf_1.CaseAlt.Cons',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='var_head_str', full_name='daml_lf_1.CaseAlt.Cons.var_head_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_head_interned_str', full_name='daml_lf_1.CaseAlt.Cons.var_head_interned_str', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_tail_str', full_name='daml_lf_1.CaseAlt.Cons.var_tail_str', index=2,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_tail_interned_str', full_name='daml_lf_1.CaseAlt.Cons.var_tail_interned_str', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='var_head', full_name='daml_lf_1.CaseAlt.Cons.var_head',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='var_tail', full_name='daml_lf_1.CaseAlt.Cons.var_tail',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=6790,
  serialized_end=6934,
)

_CASEALT_OPTIONALSOME = _descriptor.Descriptor(
  name='OptionalSome',
  full_name='daml_lf_1.CaseAlt.OptionalSome',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='var_body_str', full_name='daml_lf_1.CaseAlt.OptionalSome.var_body_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_body_interned_str', full_name='daml_lf_1.CaseAlt.OptionalSome.var_body_interned_str', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='var_body', full_name='daml_lf_1.CaseAlt.OptionalSome.var_body',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=6936,
  serialized_end=7019,
)

_CASEALT = _descriptor.Descriptor(
  name='CaseAlt',
  full_name='daml_lf_1.CaseAlt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='default', full_name='daml_lf_1.CaseAlt.default', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variant', full_name='daml_lf_1.CaseAlt.variant', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prim_con', full_name='daml_lf_1.CaseAlt.prim_con', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nil', full_name='daml_lf_1.CaseAlt.nil', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cons', full_name='daml_lf_1.CaseAlt.cons', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='optional_none', full_name='daml_lf_1.CaseAlt.optional_none', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='optional_some', full_name='daml_lf_1.CaseAlt.optional_some', index=6,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enum', full_name='daml_lf_1.CaseAlt.enum', index=7,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='daml_lf_1.CaseAlt.body', index=8,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_CASEALT_VARIANT, _CASEALT_ENUM, _CASEALT_CONS, _CASEALT_OPTIONALSOME, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='daml_lf_1.CaseAlt.Sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=6109,
  serialized_end=7026,
)


_CASE = _descriptor.Descriptor(
  name='Case',
  full_name='daml_lf_1.Case',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='scrut', full_name='daml_lf_1.Case.scrut', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='alts', full_name='daml_lf_1.Case.alts', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7028,
  serialized_end=7100,
)


_BLOCK = _descriptor.Descriptor(
  name='Block',
  full_name='daml_lf_1.Block',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bindings', full_name='daml_lf_1.Block.bindings', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='daml_lf_1.Block.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7102,
  serialized_end=7178,
)


_PURE = _descriptor.Descriptor(
  name='Pure',
  full_name='daml_lf_1.Pure',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Pure.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.Pure.expr', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7180,
  serialized_end=7248,
)


_UPDATE_CREATE = _descriptor.Descriptor(
  name='Create',
  full_name='daml_lf_1.Update.Create',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='template', full_name='daml_lf_1.Update.Create.template', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.Update.Create.expr', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7775,
  serialized_end=7856,
)

_UPDATE_EXERCISE = _descriptor.Descriptor(
  name='Exercise',
  full_name='daml_lf_1.Update.Exercise',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='template', full_name='daml_lf_1.Update.Exercise.template', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice_str', full_name='daml_lf_1.Update.Exercise.choice_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice_interned_str', full_name='daml_lf_1.Update.Exercise.choice_interned_str', index=2,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cid', full_name='daml_lf_1.Update.Exercise.cid', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arg', full_name='daml_lf_1.Update.Exercise.arg', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='choice', full_name='daml_lf_1.Update.Exercise.choice',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=7859,
  serialized_end=8040,
)

_UPDATE_EXERCISEBYKEY = _descriptor.Descriptor(
  name='ExerciseByKey',
  full_name='daml_lf_1.Update.ExerciseByKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='template', full_name='daml_lf_1.Update.ExerciseByKey.template', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice_interned_str', full_name='daml_lf_1.Update.ExerciseByKey.choice_interned_str', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='key', full_name='daml_lf_1.Update.ExerciseByKey.key', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arg', full_name='daml_lf_1.Update.ExerciseByKey.arg', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8043,
  serialized_end=8189,
)

_UPDATE_FETCH = _descriptor.Descriptor(
  name='Fetch',
  full_name='daml_lf_1.Update.Fetch',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='template', full_name='daml_lf_1.Update.Fetch.template', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cid', full_name='daml_lf_1.Update.Fetch.cid', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8191,
  serialized_end=8276,
)

_UPDATE_EMBEDEXPR = _descriptor.Descriptor(
  name='EmbedExpr',
  full_name='daml_lf_1.Update.EmbedExpr',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Update.EmbedExpr.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='daml_lf_1.Update.EmbedExpr.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8278,
  serialized_end=8351,
)

_UPDATE_RETRIEVEBYKEY = _descriptor.Descriptor(
  name='RetrieveByKey',
  full_name='daml_lf_1.Update.RetrieveByKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='template', full_name='daml_lf_1.Update.RetrieveByKey.template', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='key', full_name='daml_lf_1.Update.RetrieveByKey.key', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8353,
  serialized_end=8440,
)

_UPDATE_TRYCATCH = _descriptor.Descriptor(
  name='TryCatch',
  full_name='daml_lf_1.Update.TryCatch',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_type', full_name='daml_lf_1.Update.TryCatch.return_type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='try_expr', full_name='daml_lf_1.Update.TryCatch.try_expr', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='var_interned_str', full_name='daml_lf_1.Update.TryCatch.var_interned_str', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='catch_expr', full_name='daml_lf_1.Update.TryCatch.catch_expr', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8443,
  serialized_end=8589,
)

_UPDATE = _descriptor.Descriptor(
  name='Update',
  full_name='daml_lf_1.Update',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pure', full_name='daml_lf_1.Update.pure', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='block', full_name='daml_lf_1.Update.block', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='create', full_name='daml_lf_1.Update.create', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exercise', full_name='daml_lf_1.Update.exercise', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exercise_by_key', full_name='daml_lf_1.Update.exercise_by_key', index=4,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fetch', full_name='daml_lf_1.Update.fetch', index=5,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='get_time', full_name='daml_lf_1.Update.get_time', index=6,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lookup_by_key', full_name='daml_lf_1.Update.lookup_by_key', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fetch_by_key', full_name='daml_lf_1.Update.fetch_by_key', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='embed_expr', full_name='daml_lf_1.Update.embed_expr', index=9,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='try_catch', full_name='daml_lf_1.Update.try_catch', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_UPDATE_CREATE, _UPDATE_EXERCISE, _UPDATE_EXERCISEBYKEY, _UPDATE_FETCH, _UPDATE_EMBEDEXPR, _UPDATE_RETRIEVEBYKEY, _UPDATE_TRYCATCH, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='daml_lf_1.Update.Sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=7251,
  serialized_end=8596,
)


_SCENARIO_COMMIT = _descriptor.Descriptor(
  name='Commit',
  full_name='daml_lf_1.Scenario.Commit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='party', full_name='daml_lf_1.Scenario.Commit.party', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.Scenario.Commit.expr', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ret_type', full_name='daml_lf_1.Scenario.Commit.ret_type', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8936,
  serialized_end=9042,
)

_SCENARIO_EMBEDEXPR = _descriptor.Descriptor(
  name='EmbedExpr',
  full_name='daml_lf_1.Scenario.EmbedExpr',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.Scenario.EmbedExpr.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='daml_lf_1.Scenario.EmbedExpr.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8278,
  serialized_end=8351,
)

_SCENARIO = _descriptor.Descriptor(
  name='Scenario',
  full_name='daml_lf_1.Scenario',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pure', full_name='daml_lf_1.Scenario.pure', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='block', full_name='daml_lf_1.Scenario.block', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='commit', full_name='daml_lf_1.Scenario.commit', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mustFailAt', full_name='daml_lf_1.Scenario.mustFailAt', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pass', full_name='daml_lf_1.Scenario.pass', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='get_time', full_name='daml_lf_1.Scenario.get_time', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='get_party', full_name='daml_lf_1.Scenario.get_party', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='embed_expr', full_name='daml_lf_1.Scenario.embed_expr', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SCENARIO_COMMIT, _SCENARIO_EMBEDEXPR, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='daml_lf_1.Scenario.Sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=8599,
  serialized_end=9124,
)


_TEMPLATECHOICE = _descriptor.Descriptor(
  name='TemplateChoice',
  full_name='daml_lf_1.TemplateChoice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name_str', full_name='daml_lf_1.TemplateChoice.name_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_interned_str', full_name='daml_lf_1.TemplateChoice.name_interned_str', index=1,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='consuming', full_name='daml_lf_1.TemplateChoice.consuming', index=2,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='controllers', full_name='daml_lf_1.TemplateChoice.controllers', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observers', full_name='daml_lf_1.TemplateChoice.observers', index=4,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arg_binder', full_name='daml_lf_1.TemplateChoice.arg_binder', index=5,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ret_type', full_name='daml_lf_1.TemplateChoice.ret_type', index=6,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='update', full_name='daml_lf_1.TemplateChoice.update', index=7,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='self_binder_str', full_name='daml_lf_1.TemplateChoice.self_binder_str', index=8,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='self_binder_interned_str', full_name='daml_lf_1.TemplateChoice.self_binder_interned_str', index=9,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location', full_name='daml_lf_1.TemplateChoice.location', index=10,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='name', full_name='daml_lf_1.TemplateChoice.name',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='self_binder', full_name='daml_lf_1.TemplateChoice.self_binder',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=9127,
  serialized_end=9522,
)


_KEYEXPR_PROJECTION = _descriptor.Descriptor(
  name='Projection',
  full_name='daml_lf_1.KeyExpr.Projection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tycon', full_name='daml_lf_1.KeyExpr.Projection.tycon', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_str', full_name='daml_lf_1.KeyExpr.Projection.field_str', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_interned_str', full_name='daml_lf_1.KeyExpr.Projection.field_interned_str', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field', full_name='daml_lf_1.KeyExpr.Projection.field',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=9636,
  serialized_end=9744,
)

_KEYEXPR_PROJECTIONS = _descriptor.Descriptor(
  name='Projections',
  full_name='daml_lf_1.KeyExpr.Projections',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='projections', full_name='daml_lf_1.KeyExpr.Projections.projections', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9746,
  serialized_end=9811,
)

_KEYEXPR_RECORDFIELD = _descriptor.Descriptor(
  name='RecordField',
  full_name='daml_lf_1.KeyExpr.RecordField',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='field_str', full_name='daml_lf_1.KeyExpr.RecordField.field_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_interned_str', full_name='daml_lf_1.KeyExpr.RecordField.field_interned_str', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.KeyExpr.RecordField.expr', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field', full_name='daml_lf_1.KeyExpr.RecordField.field',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=9813,
  serialized_end=9920,
)

_KEYEXPR_RECORD = _descriptor.Descriptor(
  name='Record',
  full_name='daml_lf_1.KeyExpr.Record',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tycon', full_name='daml_lf_1.KeyExpr.Record.tycon', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fields', full_name='daml_lf_1.KeyExpr.Record.fields', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9922,
  serialized_end=10014,
)

_KEYEXPR = _descriptor.Descriptor(
  name='KeyExpr',
  full_name='daml_lf_1.KeyExpr',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='projections', full_name='daml_lf_1.KeyExpr.projections', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='record', full_name='daml_lf_1.KeyExpr.record', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_KEYEXPR_PROJECTION, _KEYEXPR_PROJECTIONS, _KEYEXPR_RECORDFIELD, _KEYEXPR_RECORD, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Sum', full_name='daml_lf_1.KeyExpr.Sum',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=9525,
  serialized_end=10021,
)


_DEFTEMPLATE_DEFKEY = _descriptor.Descriptor(
  name='DefKey',
  full_name='daml_lf_1.DefTemplate.DefKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.DefTemplate.DefKey.type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='key', full_name='daml_lf_1.DefTemplate.DefKey.key', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='complex_key', full_name='daml_lf_1.DefTemplate.DefKey.complex_key', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='maintainers', full_name='daml_lf_1.DefTemplate.DefKey.maintainers', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='key_expr', full_name='daml_lf_1.DefTemplate.DefKey.key_expr',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=10440,
  serialized_end=10604,
)

_DEFTEMPLATE = _descriptor.Descriptor(
  name='DefTemplate',
  full_name='daml_lf_1.DefTemplate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tycon_dname', full_name='daml_lf_1.DefTemplate.tycon_dname', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tycon_interned_dname', full_name='daml_lf_1.DefTemplate.tycon_interned_dname', index=1,
      number=12, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='param_str', full_name='daml_lf_1.DefTemplate.param_str', index=2,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='param_interned_str', full_name='daml_lf_1.DefTemplate.param_interned_str', index=3,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='precond', full_name='daml_lf_1.DefTemplate.precond', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='signatories', full_name='daml_lf_1.DefTemplate.signatories', index=5,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='agreement', full_name='daml_lf_1.DefTemplate.agreement', index=6,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choices', full_name='daml_lf_1.DefTemplate.choices', index=7,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observers', full_name='daml_lf_1.DefTemplate.observers', index=8,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location', full_name='daml_lf_1.DefTemplate.location', index=9,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='key', full_name='daml_lf_1.DefTemplate.key', index=10,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DEFTEMPLATE_DEFKEY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='tycon', full_name='daml_lf_1.DefTemplate.tycon',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='param', full_name='daml_lf_1.DefTemplate.param',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=10024,
  serialized_end=10628,
)


_DEFEXCEPTION = _descriptor.Descriptor(
  name='DefException',
  full_name='daml_lf_1.DefException',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name_interned_dname', full_name='daml_lf_1.DefException.name_interned_dname', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location', full_name='daml_lf_1.DefException.location', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='daml_lf_1.DefException.message', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10630,
  serialized_end=10746,
)


_DEFDATATYPE_FIELDS = _descriptor.Descriptor(
  name='Fields',
  full_name='daml_lf_1.DefDataType.Fields',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fields', full_name='daml_lf_1.DefDataType.Fields.fields', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11101,
  serialized_end=11151,
)

_DEFDATATYPE_ENUMCONSTRUCTORS = _descriptor.Descriptor(
  name='EnumConstructors',
  full_name='daml_lf_1.DefDataType.EnumConstructors',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='constructors_str', full_name='daml_lf_1.DefDataType.EnumConstructors.constructors_str', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='constructors_interned_str', full_name='daml_lf_1.DefDataType.EnumConstructors.constructors_interned_str', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11153,
  serialized_end=11232,
)

_DEFDATATYPE = _descriptor.Descriptor(
  name='DefDataType',
  full_name='daml_lf_1.DefDataType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name_dname', full_name='daml_lf_1.DefDataType.name_dname', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_interned_dname', full_name='daml_lf_1.DefDataType.name_interned_dname', index=1,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='params', full_name='daml_lf_1.DefDataType.params', index=2,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='record', full_name='daml_lf_1.DefDataType.record', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variant', full_name='daml_lf_1.DefDataType.variant', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enum', full_name='daml_lf_1.DefDataType.enum', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='serializable', full_name='daml_lf_1.DefDataType.serializable', index=6,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location', full_name='daml_lf_1.DefDataType.location', index=7,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DEFDATATYPE_FIELDS, _DEFDATATYPE_ENUMCONSTRUCTORS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='name', full_name='daml_lf_1.DefDataType.name',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='DataCons', full_name='daml_lf_1.DefDataType.DataCons',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=10749,
  serialized_end=11252,
)


_DEFTYPESYN = _descriptor.Descriptor(
  name='DefTypeSyn',
  full_name='daml_lf_1.DefTypeSyn',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name_dname', full_name='daml_lf_1.DefTypeSyn.name_dname', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_interned_dname', full_name='daml_lf_1.DefTypeSyn.name_interned_dname', index=1,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='params', full_name='daml_lf_1.DefTypeSyn.params', index=2,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.DefTypeSyn.type', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location', full_name='daml_lf_1.DefTypeSyn.location', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='name', full_name='daml_lf_1.DefTypeSyn.name',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=11255,
  serialized_end=11465,
)


_DEFVALUE_NAMEWITHTYPE = _descriptor.Descriptor(
  name='NameWithType',
  full_name='daml_lf_1.DefValue.NameWithType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name_dname', full_name='daml_lf_1.DefValue.NameWithType.name_dname', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_interned_dname', full_name='daml_lf_1.DefValue.NameWithType.name_interned_dname', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='daml_lf_1.DefValue.NameWithType.type', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11652,
  serialized_end=11746,
)

_DEFVALUE = _descriptor.Descriptor(
  name='DefValue',
  full_name='daml_lf_1.DefValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name_with_type', full_name='daml_lf_1.DefValue.name_with_type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expr', full_name='daml_lf_1.DefValue.expr', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='no_party_literals', full_name='daml_lf_1.DefValue.no_party_literals', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_test', full_name='daml_lf_1.DefValue.is_test', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location', full_name='daml_lf_1.DefValue.location', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DEFVALUE_NAMEWITHTYPE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11468,
  serialized_end=11746,
)


_FEATUREFLAGS = _descriptor.Descriptor(
  name='FeatureFlags',
  full_name='daml_lf_1.FeatureFlags',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='forbidPartyLiterals', full_name='daml_lf_1.FeatureFlags.forbidPartyLiterals', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dontDivulgeContractIdsInCreateArguments', full_name='daml_lf_1.FeatureFlags.dontDivulgeContractIdsInCreateArguments', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dontDiscloseNonConsumingChoicesToObservers', full_name='daml_lf_1.FeatureFlags.dontDiscloseNonConsumingChoicesToObservers', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11749,
  serialized_end=11893,
)


_MODULE = _descriptor.Descriptor(
  name='Module',
  full_name='daml_lf_1.Module',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name_dname', full_name='daml_lf_1.Module.name_dname', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_interned_dname', full_name='daml_lf_1.Module.name_interned_dname', index=1,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='flags', full_name='daml_lf_1.Module.flags', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='synonyms', full_name='daml_lf_1.Module.synonyms', index=3,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_types', full_name='daml_lf_1.Module.data_types', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='values', full_name='daml_lf_1.Module.values', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='templates', full_name='daml_lf_1.Module.templates', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exceptions', full_name='daml_lf_1.Module.exceptions', index=7,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='name', full_name='daml_lf_1.Module.name',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=11896,
  serialized_end=12250,
)


_INTERNEDDOTTEDNAME = _descriptor.Descriptor(
  name='InternedDottedName',
  full_name='daml_lf_1.InternedDottedName',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='segments_interned_str', full_name='daml_lf_1.InternedDottedName.segments_interned_str', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12252,
  serialized_end=12303,
)


_PACKAGEMETADATA = _descriptor.Descriptor(
  name='PackageMetadata',
  full_name='daml_lf_1.PackageMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name_interned_str', full_name='daml_lf_1.PackageMetadata.name_interned_str', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version_interned_str', full_name='daml_lf_1.PackageMetadata.version_interned_str', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12305,
  serialized_end=12379,
)


_PACKAGE = _descriptor.Descriptor(
  name='Package',
  full_name='daml_lf_1.Package',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='modules', full_name='daml_lf_1.Package.modules', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='interned_strings', full_name='daml_lf_1.Package.interned_strings', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='interned_dotted_names', full_name='daml_lf_1.Package.interned_dotted_names', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='daml_lf_1.Package.metadata', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='interned_types', full_name='daml_lf_1.Package.interned_types', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12382,
  serialized_end=12602,
)

_PACKAGEREF.fields_by_name['self'].message_type = _UNIT
_PACKAGEREF.oneofs_by_name['Sum'].fields.append(
  _PACKAGEREF.fields_by_name['self'])
_PACKAGEREF.fields_by_name['self'].containing_oneof = _PACKAGEREF.oneofs_by_name['Sum']
_PACKAGEREF.oneofs_by_name['Sum'].fields.append(
  _PACKAGEREF.fields_by_name['package_id_str'])
_PACKAGEREF.fields_by_name['package_id_str'].containing_oneof = _PACKAGEREF.oneofs_by_name['Sum']
_PACKAGEREF.oneofs_by_name['Sum'].fields.append(
  _PACKAGEREF.fields_by_name['package_id_interned_str'])
_PACKAGEREF.fields_by_name['package_id_interned_str'].containing_oneof = _PACKAGEREF.oneofs_by_name['Sum']
_MODULEREF.fields_by_name['package_ref'].message_type = _PACKAGEREF
_MODULEREF.fields_by_name['module_name_dname'].message_type = _DOTTEDNAME
_MODULEREF.oneofs_by_name['module_name'].fields.append(
  _MODULEREF.fields_by_name['module_name_dname'])
_MODULEREF.fields_by_name['module_name_dname'].containing_oneof = _MODULEREF.oneofs_by_name['module_name']
_MODULEREF.oneofs_by_name['module_name'].fields.append(
  _MODULEREF.fields_by_name['module_name_interned_dname'])
_MODULEREF.fields_by_name['module_name_interned_dname'].containing_oneof = _MODULEREF.oneofs_by_name['module_name']
_TYPECONNAME.fields_by_name['module'].message_type = _MODULEREF
_TYPECONNAME.fields_by_name['name_dname'].message_type = _DOTTEDNAME
_TYPECONNAME.oneofs_by_name['name'].fields.append(
  _TYPECONNAME.fields_by_name['name_dname'])
_TYPECONNAME.fields_by_name['name_dname'].containing_oneof = _TYPECONNAME.oneofs_by_name['name']
_TYPECONNAME.oneofs_by_name['name'].fields.append(
  _TYPECONNAME.fields_by_name['name_interned_dname'])
_TYPECONNAME.fields_by_name['name_interned_dname'].containing_oneof = _TYPECONNAME.oneofs_by_name['name']
_TYPESYNNAME.fields_by_name['module'].message_type = _MODULEREF
_TYPESYNNAME.fields_by_name['name_dname'].message_type = _DOTTEDNAME
_TYPESYNNAME.oneofs_by_name['name'].fields.append(
  _TYPESYNNAME.fields_by_name['name_dname'])
_TYPESYNNAME.fields_by_name['name_dname'].containing_oneof = _TYPESYNNAME.oneofs_by_name['name']
_TYPESYNNAME.oneofs_by_name['name'].fields.append(
  _TYPESYNNAME.fields_by_name['name_interned_dname'])
_TYPESYNNAME.fields_by_name['name_interned_dname'].containing_oneof = _TYPESYNNAME.oneofs_by_name['name']
_VALNAME.fields_by_name['module'].message_type = _MODULEREF
_FIELDWITHTYPE.fields_by_name['type'].message_type = _TYPE
_FIELDWITHTYPE.oneofs_by_name['field'].fields.append(
  _FIELDWITHTYPE.fields_by_name['field_str'])
_FIELDWITHTYPE.fields_by_name['field_str'].containing_oneof = _FIELDWITHTYPE.oneofs_by_name['field']
_FIELDWITHTYPE.oneofs_by_name['field'].fields.append(
  _FIELDWITHTYPE.fields_by_name['field_interned_str'])
_FIELDWITHTYPE.fields_by_name['field_interned_str'].containing_oneof = _FIELDWITHTYPE.oneofs_by_name['field']
_VARWITHTYPE.fields_by_name['type'].message_type = _TYPE
_VARWITHTYPE.oneofs_by_name['var'].fields.append(
  _VARWITHTYPE.fields_by_name['var_str'])
_VARWITHTYPE.fields_by_name['var_str'].containing_oneof = _VARWITHTYPE.oneofs_by_name['var']
_VARWITHTYPE.oneofs_by_name['var'].fields.append(
  _VARWITHTYPE.fields_by_name['var_interned_str'])
_VARWITHTYPE.fields_by_name['var_interned_str'].containing_oneof = _VARWITHTYPE.oneofs_by_name['var']
_TYPEVARWITHKIND.fields_by_name['kind'].message_type = _KIND
_TYPEVARWITHKIND.oneofs_by_name['var'].fields.append(
  _TYPEVARWITHKIND.fields_by_name['var_str'])
_TYPEVARWITHKIND.fields_by_name['var_str'].containing_oneof = _TYPEVARWITHKIND.oneofs_by_name['var']
_TYPEVARWITHKIND.oneofs_by_name['var'].fields.append(
  _TYPEVARWITHKIND.fields_by_name['var_interned_str'])
_TYPEVARWITHKIND.fields_by_name['var_interned_str'].containing_oneof = _TYPEVARWITHKIND.oneofs_by_name['var']
_FIELDWITHEXPR.fields_by_name['expr'].message_type = _EXPR
_FIELDWITHEXPR.oneofs_by_name['field'].fields.append(
  _FIELDWITHEXPR.fields_by_name['field_str'])
_FIELDWITHEXPR.fields_by_name['field_str'].containing_oneof = _FIELDWITHEXPR.oneofs_by_name['field']
_FIELDWITHEXPR.oneofs_by_name['field'].fields.append(
  _FIELDWITHEXPR.fields_by_name['field_interned_str'])
_FIELDWITHEXPR.fields_by_name['field_interned_str'].containing_oneof = _FIELDWITHEXPR.oneofs_by_name['field']
_BINDING.fields_by_name['binder'].message_type = _VARWITHTYPE
_BINDING.fields_by_name['bound'].message_type = _EXPR
_KIND_ARROW.fields_by_name['params'].message_type = _KIND
_KIND_ARROW.fields_by_name['result'].message_type = _KIND
_KIND_ARROW.containing_type = _KIND
_KIND.fields_by_name['star'].message_type = _UNIT
_KIND.fields_by_name['arrow'].message_type = _KIND_ARROW
_KIND.fields_by_name['nat'].message_type = _UNIT
_KIND.oneofs_by_name['Sum'].fields.append(
  _KIND.fields_by_name['star'])
_KIND.fields_by_name['star'].containing_oneof = _KIND.oneofs_by_name['Sum']
_KIND.oneofs_by_name['Sum'].fields.append(
  _KIND.fields_by_name['arrow'])
_KIND.fields_by_name['arrow'].containing_oneof = _KIND.oneofs_by_name['Sum']
_KIND.oneofs_by_name['Sum'].fields.append(
  _KIND.fields_by_name['nat'])
_KIND.fields_by_name['nat'].containing_oneof = _KIND.oneofs_by_name['Sum']
_TYPE_VAR.fields_by_name['args'].message_type = _TYPE
_TYPE_VAR.containing_type = _TYPE
_TYPE_VAR.oneofs_by_name['var'].fields.append(
  _TYPE_VAR.fields_by_name['var_str'])
_TYPE_VAR.fields_by_name['var_str'].containing_oneof = _TYPE_VAR.oneofs_by_name['var']
_TYPE_VAR.oneofs_by_name['var'].fields.append(
  _TYPE_VAR.fields_by_name['var_interned_str'])
_TYPE_VAR.fields_by_name['var_interned_str'].containing_oneof = _TYPE_VAR.oneofs_by_name['var']
_TYPE_CON.fields_by_name['tycon'].message_type = _TYPECONNAME
_TYPE_CON.fields_by_name['args'].message_type = _TYPE
_TYPE_CON.containing_type = _TYPE
_TYPE_SYN.fields_by_name['tysyn'].message_type = _TYPESYNNAME
_TYPE_SYN.fields_by_name['args'].message_type = _TYPE
_TYPE_SYN.containing_type = _TYPE
_TYPE_PRIM.fields_by_name['prim'].enum_type = _PRIMTYPE
_TYPE_PRIM.fields_by_name['args'].message_type = _TYPE
_TYPE_PRIM.containing_type = _TYPE
_TYPE_FUN.fields_by_name['params'].message_type = _TYPE
_TYPE_FUN.fields_by_name['result'].message_type = _TYPE
_TYPE_FUN.containing_type = _TYPE
_TYPE_FORALL.fields_by_name['vars'].message_type = _TYPEVARWITHKIND
_TYPE_FORALL.fields_by_name['body'].message_type = _TYPE
_TYPE_FORALL.containing_type = _TYPE
_TYPE_STRUCT.fields_by_name['fields'].message_type = _FIELDWITHTYPE
_TYPE_STRUCT.containing_type = _TYPE
_TYPE.fields_by_name['var'].message_type = _TYPE_VAR
_TYPE.fields_by_name['con'].message_type = _TYPE_CON
_TYPE.fields_by_name['prim'].message_type = _TYPE_PRIM
_TYPE.fields_by_name['forall'].message_type = _TYPE_FORALL
_TYPE.fields_by_name['struct'].message_type = _TYPE_STRUCT
_TYPE.fields_by_name['syn'].message_type = _TYPE_SYN
_TYPE.oneofs_by_name['Sum'].fields.append(
  _TYPE.fields_by_name['var'])
_TYPE.fields_by_name['var'].containing_oneof = _TYPE.oneofs_by_name['Sum']
_TYPE.oneofs_by_name['Sum'].fields.append(
  _TYPE.fields_by_name['con'])
_TYPE.fields_by_name['con'].containing_oneof = _TYPE.oneofs_by_name['Sum']
_TYPE.oneofs_by_name['Sum'].fields.append(
  _TYPE.fields_by_name['prim'])
_TYPE.fields_by_name['prim'].containing_oneof = _TYPE.oneofs_by_name['Sum']
_TYPE.oneofs_by_name['Sum'].fields.append(
  _TYPE.fields_by_name['forall'])
_TYPE.fields_by_name['forall'].containing_oneof = _TYPE.oneofs_by_name['Sum']
_TYPE.oneofs_by_name['Sum'].fields.append(
  _TYPE.fields_by_name['struct'])
_TYPE.fields_by_name['struct'].containing_oneof = _TYPE.oneofs_by_name['Sum']
_TYPE.oneofs_by_name['Sum'].fields.append(
  _TYPE.fields_by_name['nat'])
_TYPE.fields_by_name['nat'].containing_oneof = _TYPE.oneofs_by_name['Sum']
_TYPE.oneofs_by_name['Sum'].fields.append(
  _TYPE.fields_by_name['syn'])
_TYPE.fields_by_name['syn'].containing_oneof = _TYPE.oneofs_by_name['Sum']
_TYPE.oneofs_by_name['Sum'].fields.append(
  _TYPE.fields_by_name['interned'])
_TYPE.fields_by_name['interned'].containing_oneof = _TYPE.oneofs_by_name['Sum']
_PRIMLIT.oneofs_by_name['Sum'].fields.append(
  _PRIMLIT.fields_by_name['int64'])
_PRIMLIT.fields_by_name['int64'].containing_oneof = _PRIMLIT.oneofs_by_name['Sum']
_PRIMLIT.oneofs_by_name['Sum'].fields.append(
  _PRIMLIT.fields_by_name['decimal_str'])
_PRIMLIT.fields_by_name['decimal_str'].containing_oneof = _PRIMLIT.oneofs_by_name['Sum']
_PRIMLIT.oneofs_by_name['Sum'].fields.append(
  _PRIMLIT.fields_by_name['numeric_interned_str'])
_PRIMLIT.fields_by_name['numeric_interned_str'].containing_oneof = _PRIMLIT.oneofs_by_name['Sum']
_PRIMLIT.oneofs_by_name['Sum'].fields.append(
  _PRIMLIT.fields_by_name['text_str'])
_PRIMLIT.fields_by_name['text_str'].containing_oneof = _PRIMLIT.oneofs_by_name['Sum']
_PRIMLIT.oneofs_by_name['Sum'].fields.append(
  _PRIMLIT.fields_by_name['text_interned_str'])
_PRIMLIT.fields_by_name['text_interned_str'].containing_oneof = _PRIMLIT.oneofs_by_name['Sum']
_PRIMLIT.oneofs_by_name['Sum'].fields.append(
  _PRIMLIT.fields_by_name['timestamp'])
_PRIMLIT.fields_by_name['timestamp'].containing_oneof = _PRIMLIT.oneofs_by_name['Sum']
_PRIMLIT.oneofs_by_name['Sum'].fields.append(
  _PRIMLIT.fields_by_name['party_str'])
_PRIMLIT.fields_by_name['party_str'].containing_oneof = _PRIMLIT.oneofs_by_name['Sum']
_PRIMLIT.oneofs_by_name['Sum'].fields.append(
  _PRIMLIT.fields_by_name['party_interned_str'])
_PRIMLIT.fields_by_name['party_interned_str'].containing_oneof = _PRIMLIT.oneofs_by_name['Sum']
_PRIMLIT.oneofs_by_name['Sum'].fields.append(
  _PRIMLIT.fields_by_name['date'])
_PRIMLIT.fields_by_name['date'].containing_oneof = _PRIMLIT.oneofs_by_name['Sum']
_LOCATION_RANGE.containing_type = _LOCATION
_LOCATION.fields_by_name['module'].message_type = _MODULEREF
_LOCATION.fields_by_name['range'].message_type = _LOCATION_RANGE
_EXPR_RECCON.fields_by_name['tycon'].message_type = _TYPE_CON
_EXPR_RECCON.fields_by_name['fields'].message_type = _FIELDWITHEXPR
_EXPR_RECCON.containing_type = _EXPR
_EXPR_RECPROJ.fields_by_name['tycon'].message_type = _TYPE_CON
_EXPR_RECPROJ.fields_by_name['record'].message_type = _EXPR
_EXPR_RECPROJ.containing_type = _EXPR
_EXPR_RECPROJ.oneofs_by_name['field'].fields.append(
  _EXPR_RECPROJ.fields_by_name['field_str'])
_EXPR_RECPROJ.fields_by_name['field_str'].containing_oneof = _EXPR_RECPROJ.oneofs_by_name['field']
_EXPR_RECPROJ.oneofs_by_name['field'].fields.append(
  _EXPR_RECPROJ.fields_by_name['field_interned_str'])
_EXPR_RECPROJ.fields_by_name['field_interned_str'].containing_oneof = _EXPR_RECPROJ.oneofs_by_name['field']
_EXPR_RECUPD.fields_by_name['tycon'].message_type = _TYPE_CON
_EXPR_RECUPD.fields_by_name['record'].message_type = _EXPR
_EXPR_RECUPD.fields_by_name['update'].message_type = _EXPR
_EXPR_RECUPD.containing_type = _EXPR
_EXPR_RECUPD.oneofs_by_name['field'].fields.append(
  _EXPR_RECUPD.fields_by_name['field_str'])
_EXPR_RECUPD.fields_by_name['field_str'].containing_oneof = _EXPR_RECUPD.oneofs_by_name['field']
_EXPR_RECUPD.oneofs_by_name['field'].fields.append(
  _EXPR_RECUPD.fields_by_name['field_interned_str'])
_EXPR_RECUPD.fields_by_name['field_interned_str'].containing_oneof = _EXPR_RECUPD.oneofs_by_name['field']
_EXPR_VARIANTCON.fields_by_name['tycon'].message_type = _TYPE_CON
_EXPR_VARIANTCON.fields_by_name['variant_arg'].message_type = _EXPR
_EXPR_VARIANTCON.containing_type = _EXPR
_EXPR_VARIANTCON.oneofs_by_name['variant_con'].fields.append(
  _EXPR_VARIANTCON.fields_by_name['variant_con_str'])
_EXPR_VARIANTCON.fields_by_name['variant_con_str'].containing_oneof = _EXPR_VARIANTCON.oneofs_by_name['variant_con']
_EXPR_VARIANTCON.oneofs_by_name['variant_con'].fields.append(
  _EXPR_VARIANTCON.fields_by_name['variant_con_interned_str'])
_EXPR_VARIANTCON.fields_by_name['variant_con_interned_str'].containing_oneof = _EXPR_VARIANTCON.oneofs_by_name['variant_con']
_EXPR_ENUMCON.fields_by_name['tycon'].message_type = _TYPECONNAME
_EXPR_ENUMCON.containing_type = _EXPR
_EXPR_ENUMCON.oneofs_by_name['enum_con'].fields.append(
  _EXPR_ENUMCON.fields_by_name['enum_con_str'])
_EXPR_ENUMCON.fields_by_name['enum_con_str'].containing_oneof = _EXPR_ENUMCON.oneofs_by_name['enum_con']
_EXPR_ENUMCON.oneofs_by_name['enum_con'].fields.append(
  _EXPR_ENUMCON.fields_by_name['enum_con_interned_str'])
_EXPR_ENUMCON.fields_by_name['enum_con_interned_str'].containing_oneof = _EXPR_ENUMCON.oneofs_by_name['enum_con']
_EXPR_STRUCTCON.fields_by_name['fields'].message_type = _FIELDWITHEXPR
_EXPR_STRUCTCON.containing_type = _EXPR
_EXPR_STRUCTPROJ.fields_by_name['struct'].message_type = _EXPR
_EXPR_STRUCTPROJ.containing_type = _EXPR
_EXPR_STRUCTPROJ.oneofs_by_name['field'].fields.append(
  _EXPR_STRUCTPROJ.fields_by_name['field_str'])
_EXPR_STRUCTPROJ.fields_by_name['field_str'].containing_oneof = _EXPR_STRUCTPROJ.oneofs_by_name['field']
_EXPR_STRUCTPROJ.oneofs_by_name['field'].fields.append(
  _EXPR_STRUCTPROJ.fields_by_name['field_interned_str'])
_EXPR_STRUCTPROJ.fields_by_name['field_interned_str'].containing_oneof = _EXPR_STRUCTPROJ.oneofs_by_name['field']
_EXPR_STRUCTUPD.fields_by_name['struct'].message_type = _EXPR
_EXPR_STRUCTUPD.fields_by_name['update'].message_type = _EXPR
_EXPR_STRUCTUPD.containing_type = _EXPR
_EXPR_STRUCTUPD.oneofs_by_name['field'].fields.append(
  _EXPR_STRUCTUPD.fields_by_name['field_str'])
_EXPR_STRUCTUPD.fields_by_name['field_str'].containing_oneof = _EXPR_STRUCTUPD.oneofs_by_name['field']
_EXPR_STRUCTUPD.oneofs_by_name['field'].fields.append(
  _EXPR_STRUCTUPD.fields_by_name['field_interned_str'])
_EXPR_STRUCTUPD.fields_by_name['field_interned_str'].containing_oneof = _EXPR_STRUCTUPD.oneofs_by_name['field']
_EXPR_APP.fields_by_name['fun'].message_type = _EXPR
_EXPR_APP.fields_by_name['args'].message_type = _EXPR
_EXPR_APP.containing_type = _EXPR
_EXPR_TYAPP.fields_by_name['expr'].message_type = _EXPR
_EXPR_TYAPP.fields_by_name['types'].message_type = _TYPE
_EXPR_TYAPP.containing_type = _EXPR
_EXPR_ABS.fields_by_name['param'].message_type = _VARWITHTYPE
_EXPR_ABS.fields_by_name['body'].message_type = _EXPR
_EXPR_ABS.containing_type = _EXPR
_EXPR_TYABS.fields_by_name['param'].message_type = _TYPEVARWITHKIND
_EXPR_TYABS.fields_by_name['body'].message_type = _EXPR
_EXPR_TYABS.containing_type = _EXPR
_EXPR_NIL.fields_by_name['type'].message_type = _TYPE
_EXPR_NIL.containing_type = _EXPR
_EXPR_CONS.fields_by_name['type'].message_type = _TYPE
_EXPR_CONS.fields_by_name['front'].message_type = _EXPR
_EXPR_CONS.fields_by_name['tail'].message_type = _EXPR
_EXPR_CONS.containing_type = _EXPR
_EXPR_OPTIONALNONE.fields_by_name['type'].message_type = _TYPE
_EXPR_OPTIONALNONE.containing_type = _EXPR
_EXPR_OPTIONALSOME.fields_by_name['type'].message_type = _TYPE
_EXPR_OPTIONALSOME.fields_by_name['body'].message_type = _EXPR
_EXPR_OPTIONALSOME.containing_type = _EXPR
_EXPR_TOANY.fields_by_name['type'].message_type = _TYPE
_EXPR_TOANY.fields_by_name['expr'].message_type = _EXPR
_EXPR_TOANY.containing_type = _EXPR
_EXPR_FROMANY.fields_by_name['type'].message_type = _TYPE
_EXPR_FROMANY.fields_by_name['expr'].message_type = _EXPR
_EXPR_FROMANY.containing_type = _EXPR
_EXPR_TOANYEXCEPTION.fields_by_name['type'].message_type = _TYPE
_EXPR_TOANYEXCEPTION.fields_by_name['expr'].message_type = _EXPR
_EXPR_TOANYEXCEPTION.containing_type = _EXPR
_EXPR_FROMANYEXCEPTION.fields_by_name['type'].message_type = _TYPE
_EXPR_FROMANYEXCEPTION.fields_by_name['expr'].message_type = _EXPR
_EXPR_FROMANYEXCEPTION.containing_type = _EXPR
_EXPR_THROW.fields_by_name['return_type'].message_type = _TYPE
_EXPR_THROW.fields_by_name['exception_type'].message_type = _TYPE
_EXPR_THROW.fields_by_name['exception_expr'].message_type = _EXPR
_EXPR_THROW.containing_type = _EXPR
_EXPR.fields_by_name['location'].message_type = _LOCATION
_EXPR.fields_by_name['val'].message_type = _VALNAME
_EXPR.fields_by_name['builtin'].enum_type = _BUILTINFUNCTION
_EXPR.fields_by_name['prim_con'].enum_type = _PRIMCON
_EXPR.fields_by_name['prim_lit'].message_type = _PRIMLIT
_EXPR.fields_by_name['rec_con'].message_type = _EXPR_RECCON
_EXPR.fields_by_name['rec_proj'].message_type = _EXPR_RECPROJ
_EXPR.fields_by_name['rec_upd'].message_type = _EXPR_RECUPD
_EXPR.fields_by_name['variant_con'].message_type = _EXPR_VARIANTCON
_EXPR.fields_by_name['enum_con'].message_type = _EXPR_ENUMCON
_EXPR.fields_by_name['struct_con'].message_type = _EXPR_STRUCTCON
_EXPR.fields_by_name['struct_proj'].message_type = _EXPR_STRUCTPROJ
_EXPR.fields_by_name['struct_upd'].message_type = _EXPR_STRUCTUPD
_EXPR.fields_by_name['app'].message_type = _EXPR_APP
_EXPR.fields_by_name['ty_app'].message_type = _EXPR_TYAPP
_EXPR.fields_by_name['abs'].message_type = _EXPR_ABS
_EXPR.fields_by_name['ty_abs'].message_type = _EXPR_TYABS
_EXPR.fields_by_name['case'].message_type = _CASE
_EXPR.fields_by_name['let'].message_type = _BLOCK
_EXPR.fields_by_name['nil'].message_type = _EXPR_NIL
_EXPR.fields_by_name['cons'].message_type = _EXPR_CONS
_EXPR.fields_by_name['update'].message_type = _UPDATE
_EXPR.fields_by_name['scenario'].message_type = _SCENARIO
_EXPR.fields_by_name['optional_none'].message_type = _EXPR_OPTIONALNONE
_EXPR.fields_by_name['optional_some'].message_type = _EXPR_OPTIONALSOME
_EXPR.fields_by_name['to_any'].message_type = _EXPR_TOANY
_EXPR.fields_by_name['from_any'].message_type = _EXPR_FROMANY
_EXPR.fields_by_name['type_rep'].message_type = _TYPE
_EXPR.fields_by_name['to_any_exception'].message_type = _EXPR_TOANYEXCEPTION
_EXPR.fields_by_name['from_any_exception'].message_type = _EXPR_FROMANYEXCEPTION
_EXPR.fields_by_name['throw'].message_type = _EXPR_THROW
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['var_str'])
_EXPR.fields_by_name['var_str'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['var_interned_str'])
_EXPR.fields_by_name['var_interned_str'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['val'])
_EXPR.fields_by_name['val'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['builtin'])
_EXPR.fields_by_name['builtin'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['prim_con'])
_EXPR.fields_by_name['prim_con'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['prim_lit'])
_EXPR.fields_by_name['prim_lit'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['rec_con'])
_EXPR.fields_by_name['rec_con'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['rec_proj'])
_EXPR.fields_by_name['rec_proj'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['rec_upd'])
_EXPR.fields_by_name['rec_upd'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['variant_con'])
_EXPR.fields_by_name['variant_con'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['enum_con'])
_EXPR.fields_by_name['enum_con'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['struct_con'])
_EXPR.fields_by_name['struct_con'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['struct_proj'])
_EXPR.fields_by_name['struct_proj'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['struct_upd'])
_EXPR.fields_by_name['struct_upd'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['app'])
_EXPR.fields_by_name['app'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['ty_app'])
_EXPR.fields_by_name['ty_app'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['abs'])
_EXPR.fields_by_name['abs'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['ty_abs'])
_EXPR.fields_by_name['ty_abs'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['case'])
_EXPR.fields_by_name['case'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['let'])
_EXPR.fields_by_name['let'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['nil'])
_EXPR.fields_by_name['nil'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['cons'])
_EXPR.fields_by_name['cons'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['update'])
_EXPR.fields_by_name['update'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['scenario'])
_EXPR.fields_by_name['scenario'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['optional_none'])
_EXPR.fields_by_name['optional_none'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['optional_some'])
_EXPR.fields_by_name['optional_some'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['to_any'])
_EXPR.fields_by_name['to_any'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['from_any'])
_EXPR.fields_by_name['from_any'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['type_rep'])
_EXPR.fields_by_name['type_rep'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['to_any_exception'])
_EXPR.fields_by_name['to_any_exception'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['from_any_exception'])
_EXPR.fields_by_name['from_any_exception'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_EXPR.oneofs_by_name['Sum'].fields.append(
  _EXPR.fields_by_name['throw'])
_EXPR.fields_by_name['throw'].containing_oneof = _EXPR.oneofs_by_name['Sum']
_CASEALT_VARIANT.fields_by_name['con'].message_type = _TYPECONNAME
_CASEALT_VARIANT.containing_type = _CASEALT
_CASEALT_VARIANT.oneofs_by_name['variant'].fields.append(
  _CASEALT_VARIANT.fields_by_name['variant_str'])
_CASEALT_VARIANT.fields_by_name['variant_str'].containing_oneof = _CASEALT_VARIANT.oneofs_by_name['variant']
_CASEALT_VARIANT.oneofs_by_name['variant'].fields.append(
  _CASEALT_VARIANT.fields_by_name['variant_interned_str'])
_CASEALT_VARIANT.fields_by_name['variant_interned_str'].containing_oneof = _CASEALT_VARIANT.oneofs_by_name['variant']
_CASEALT_VARIANT.oneofs_by_name['binder'].fields.append(
  _CASEALT_VARIANT.fields_by_name['binder_str'])
_CASEALT_VARIANT.fields_by_name['binder_str'].containing_oneof = _CASEALT_VARIANT.oneofs_by_name['binder']
_CASEALT_VARIANT.oneofs_by_name['binder'].fields.append(
  _CASEALT_VARIANT.fields_by_name['binder_interned_str'])
_CASEALT_VARIANT.fields_by_name['binder_interned_str'].containing_oneof = _CASEALT_VARIANT.oneofs_by_name['binder']
_CASEALT_ENUM.fields_by_name['con'].message_type = _TYPECONNAME
_CASEALT_ENUM.containing_type = _CASEALT
_CASEALT_ENUM.oneofs_by_name['constructor'].fields.append(
  _CASEALT_ENUM.fields_by_name['constructor_str'])
_CASEALT_ENUM.fields_by_name['constructor_str'].containing_oneof = _CASEALT_ENUM.oneofs_by_name['constructor']
_CASEALT_ENUM.oneofs_by_name['constructor'].fields.append(
  _CASEALT_ENUM.fields_by_name['constructor_interned_str'])
_CASEALT_ENUM.fields_by_name['constructor_interned_str'].containing_oneof = _CASEALT_ENUM.oneofs_by_name['constructor']
_CASEALT_CONS.containing_type = _CASEALT
_CASEALT_CONS.oneofs_by_name['var_head'].fields.append(
  _CASEALT_CONS.fields_by_name['var_head_str'])
_CASEALT_CONS.fields_by_name['var_head_str'].containing_oneof = _CASEALT_CONS.oneofs_by_name['var_head']
_CASEALT_CONS.oneofs_by_name['var_head'].fields.append(
  _CASEALT_CONS.fields_by_name['var_head_interned_str'])
_CASEALT_CONS.fields_by_name['var_head_interned_str'].containing_oneof = _CASEALT_CONS.oneofs_by_name['var_head']
_CASEALT_CONS.oneofs_by_name['var_tail'].fields.append(
  _CASEALT_CONS.fields_by_name['var_tail_str'])
_CASEALT_CONS.fields_by_name['var_tail_str'].containing_oneof = _CASEALT_CONS.oneofs_by_name['var_tail']
_CASEALT_CONS.oneofs_by_name['var_tail'].fields.append(
  _CASEALT_CONS.fields_by_name['var_tail_interned_str'])
_CASEALT_CONS.fields_by_name['var_tail_interned_str'].containing_oneof = _CASEALT_CONS.oneofs_by_name['var_tail']
_CASEALT_OPTIONALSOME.containing_type = _CASEALT
_CASEALT_OPTIONALSOME.oneofs_by_name['var_body'].fields.append(
  _CASEALT_OPTIONALSOME.fields_by_name['var_body_str'])
_CASEALT_OPTIONALSOME.fields_by_name['var_body_str'].containing_oneof = _CASEALT_OPTIONALSOME.oneofs_by_name['var_body']
_CASEALT_OPTIONALSOME.oneofs_by_name['var_body'].fields.append(
  _CASEALT_OPTIONALSOME.fields_by_name['var_body_interned_str'])
_CASEALT_OPTIONALSOME.fields_by_name['var_body_interned_str'].containing_oneof = _CASEALT_OPTIONALSOME.oneofs_by_name['var_body']
_CASEALT.fields_by_name['default'].message_type = _UNIT
_CASEALT.fields_by_name['variant'].message_type = _CASEALT_VARIANT
_CASEALT.fields_by_name['prim_con'].enum_type = _PRIMCON
_CASEALT.fields_by_name['nil'].message_type = _UNIT
_CASEALT.fields_by_name['cons'].message_type = _CASEALT_CONS
_CASEALT.fields_by_name['optional_none'].message_type = _UNIT
_CASEALT.fields_by_name['optional_some'].message_type = _CASEALT_OPTIONALSOME
_CASEALT.fields_by_name['enum'].message_type = _CASEALT_ENUM
_CASEALT.fields_by_name['body'].message_type = _EXPR
_CASEALT.oneofs_by_name['Sum'].fields.append(
  _CASEALT.fields_by_name['default'])
_CASEALT.fields_by_name['default'].containing_oneof = _CASEALT.oneofs_by_name['Sum']
_CASEALT.oneofs_by_name['Sum'].fields.append(
  _CASEALT.fields_by_name['variant'])
_CASEALT.fields_by_name['variant'].containing_oneof = _CASEALT.oneofs_by_name['Sum']
_CASEALT.oneofs_by_name['Sum'].fields.append(
  _CASEALT.fields_by_name['prim_con'])
_CASEALT.fields_by_name['prim_con'].containing_oneof = _CASEALT.oneofs_by_name['Sum']
_CASEALT.oneofs_by_name['Sum'].fields.append(
  _CASEALT.fields_by_name['nil'])
_CASEALT.fields_by_name['nil'].containing_oneof = _CASEALT.oneofs_by_name['Sum']
_CASEALT.oneofs_by_name['Sum'].fields.append(
  _CASEALT.fields_by_name['cons'])
_CASEALT.fields_by_name['cons'].containing_oneof = _CASEALT.oneofs_by_name['Sum']
_CASEALT.oneofs_by_name['Sum'].fields.append(
  _CASEALT.fields_by_name['optional_none'])
_CASEALT.fields_by_name['optional_none'].containing_oneof = _CASEALT.oneofs_by_name['Sum']
_CASEALT.oneofs_by_name['Sum'].fields.append(
  _CASEALT.fields_by_name['optional_some'])
_CASEALT.fields_by_name['optional_some'].containing_oneof = _CASEALT.oneofs_by_name['Sum']
_CASEALT.oneofs_by_name['Sum'].fields.append(
  _CASEALT.fields_by_name['enum'])
_CASEALT.fields_by_name['enum'].containing_oneof = _CASEALT.oneofs_by_name['Sum']
_CASE.fields_by_name['scrut'].message_type = _EXPR
_CASE.fields_by_name['alts'].message_type = _CASEALT
_BLOCK.fields_by_name['bindings'].message_type = _BINDING
_BLOCK.fields_by_name['body'].message_type = _EXPR
_PURE.fields_by_name['type'].message_type = _TYPE
_PURE.fields_by_name['expr'].message_type = _EXPR
_UPDATE_CREATE.fields_by_name['template'].message_type = _TYPECONNAME
_UPDATE_CREATE.fields_by_name['expr'].message_type = _EXPR
_UPDATE_CREATE.containing_type = _UPDATE
_UPDATE_EXERCISE.fields_by_name['template'].message_type = _TYPECONNAME
_UPDATE_EXERCISE.fields_by_name['cid'].message_type = _EXPR
_UPDATE_EXERCISE.fields_by_name['arg'].message_type = _EXPR
_UPDATE_EXERCISE.containing_type = _UPDATE
_UPDATE_EXERCISE.oneofs_by_name['choice'].fields.append(
  _UPDATE_EXERCISE.fields_by_name['choice_str'])
_UPDATE_EXERCISE.fields_by_name['choice_str'].containing_oneof = _UPDATE_EXERCISE.oneofs_by_name['choice']
_UPDATE_EXERCISE.oneofs_by_name['choice'].fields.append(
  _UPDATE_EXERCISE.fields_by_name['choice_interned_str'])
_UPDATE_EXERCISE.fields_by_name['choice_interned_str'].containing_oneof = _UPDATE_EXERCISE.oneofs_by_name['choice']
_UPDATE_EXERCISEBYKEY.fields_by_name['template'].message_type = _TYPECONNAME
_UPDATE_EXERCISEBYKEY.fields_by_name['key'].message_type = _EXPR
_UPDATE_EXERCISEBYKEY.fields_by_name['arg'].message_type = _EXPR
_UPDATE_EXERCISEBYKEY.containing_type = _UPDATE
_UPDATE_FETCH.fields_by_name['template'].message_type = _TYPECONNAME
_UPDATE_FETCH.fields_by_name['cid'].message_type = _EXPR
_UPDATE_FETCH.containing_type = _UPDATE
_UPDATE_EMBEDEXPR.fields_by_name['type'].message_type = _TYPE
_UPDATE_EMBEDEXPR.fields_by_name['body'].message_type = _EXPR
_UPDATE_EMBEDEXPR.containing_type = _UPDATE
_UPDATE_RETRIEVEBYKEY.fields_by_name['template'].message_type = _TYPECONNAME
_UPDATE_RETRIEVEBYKEY.fields_by_name['key'].message_type = _EXPR
_UPDATE_RETRIEVEBYKEY.containing_type = _UPDATE
_UPDATE_TRYCATCH.fields_by_name['return_type'].message_type = _TYPE
_UPDATE_TRYCATCH.fields_by_name['try_expr'].message_type = _EXPR
_UPDATE_TRYCATCH.fields_by_name['catch_expr'].message_type = _EXPR
_UPDATE_TRYCATCH.containing_type = _UPDATE
_UPDATE.fields_by_name['pure'].message_type = _PURE
_UPDATE.fields_by_name['block'].message_type = _BLOCK
_UPDATE.fields_by_name['create'].message_type = _UPDATE_CREATE
_UPDATE.fields_by_name['exercise'].message_type = _UPDATE_EXERCISE
_UPDATE.fields_by_name['exercise_by_key'].message_type = _UPDATE_EXERCISEBYKEY
_UPDATE.fields_by_name['fetch'].message_type = _UPDATE_FETCH
_UPDATE.fields_by_name['get_time'].message_type = _UNIT
_UPDATE.fields_by_name['lookup_by_key'].message_type = _UPDATE_RETRIEVEBYKEY
_UPDATE.fields_by_name['fetch_by_key'].message_type = _UPDATE_RETRIEVEBYKEY
_UPDATE.fields_by_name['embed_expr'].message_type = _UPDATE_EMBEDEXPR
_UPDATE.fields_by_name['try_catch'].message_type = _UPDATE_TRYCATCH
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['pure'])
_UPDATE.fields_by_name['pure'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['block'])
_UPDATE.fields_by_name['block'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['create'])
_UPDATE.fields_by_name['create'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['exercise'])
_UPDATE.fields_by_name['exercise'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['exercise_by_key'])
_UPDATE.fields_by_name['exercise_by_key'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['fetch'])
_UPDATE.fields_by_name['fetch'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['get_time'])
_UPDATE.fields_by_name['get_time'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['lookup_by_key'])
_UPDATE.fields_by_name['lookup_by_key'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['fetch_by_key'])
_UPDATE.fields_by_name['fetch_by_key'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['embed_expr'])
_UPDATE.fields_by_name['embed_expr'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_UPDATE.oneofs_by_name['Sum'].fields.append(
  _UPDATE.fields_by_name['try_catch'])
_UPDATE.fields_by_name['try_catch'].containing_oneof = _UPDATE.oneofs_by_name['Sum']
_SCENARIO_COMMIT.fields_by_name['party'].message_type = _EXPR
_SCENARIO_COMMIT.fields_by_name['expr'].message_type = _EXPR
_SCENARIO_COMMIT.fields_by_name['ret_type'].message_type = _TYPE
_SCENARIO_COMMIT.containing_type = _SCENARIO
_SCENARIO_EMBEDEXPR.fields_by_name['type'].message_type = _TYPE
_SCENARIO_EMBEDEXPR.fields_by_name['body'].message_type = _EXPR
_SCENARIO_EMBEDEXPR.containing_type = _SCENARIO
_SCENARIO.fields_by_name['pure'].message_type = _PURE
_SCENARIO.fields_by_name['block'].message_type = _BLOCK
_SCENARIO.fields_by_name['commit'].message_type = _SCENARIO_COMMIT
_SCENARIO.fields_by_name['mustFailAt'].message_type = _SCENARIO_COMMIT
_SCENARIO.fields_by_name['pass'].message_type = _EXPR
_SCENARIO.fields_by_name['get_time'].message_type = _UNIT
_SCENARIO.fields_by_name['get_party'].message_type = _EXPR
_SCENARIO.fields_by_name['embed_expr'].message_type = _SCENARIO_EMBEDEXPR
_SCENARIO.oneofs_by_name['Sum'].fields.append(
  _SCENARIO.fields_by_name['pure'])
_SCENARIO.fields_by_name['pure'].containing_oneof = _SCENARIO.oneofs_by_name['Sum']
_SCENARIO.oneofs_by_name['Sum'].fields.append(
  _SCENARIO.fields_by_name['block'])
_SCENARIO.fields_by_name['block'].containing_oneof = _SCENARIO.oneofs_by_name['Sum']
_SCENARIO.oneofs_by_name['Sum'].fields.append(
  _SCENARIO.fields_by_name['commit'])
_SCENARIO.fields_by_name['commit'].containing_oneof = _SCENARIO.oneofs_by_name['Sum']
_SCENARIO.oneofs_by_name['Sum'].fields.append(
  _SCENARIO.fields_by_name['mustFailAt'])
_SCENARIO.fields_by_name['mustFailAt'].containing_oneof = _SCENARIO.oneofs_by_name['Sum']
_SCENARIO.oneofs_by_name['Sum'].fields.append(
  _SCENARIO.fields_by_name['pass'])
_SCENARIO.fields_by_name['pass'].containing_oneof = _SCENARIO.oneofs_by_name['Sum']
_SCENARIO.oneofs_by_name['Sum'].fields.append(
  _SCENARIO.fields_by_name['get_time'])
_SCENARIO.fields_by_name['get_time'].containing_oneof = _SCENARIO.oneofs_by_name['Sum']
_SCENARIO.oneofs_by_name['Sum'].fields.append(
  _SCENARIO.fields_by_name['get_party'])
_SCENARIO.fields_by_name['get_party'].containing_oneof = _SCENARIO.oneofs_by_name['Sum']
_SCENARIO.oneofs_by_name['Sum'].fields.append(
  _SCENARIO.fields_by_name['embed_expr'])
_SCENARIO.fields_by_name['embed_expr'].containing_oneof = _SCENARIO.oneofs_by_name['Sum']
_TEMPLATECHOICE.fields_by_name['controllers'].message_type = _EXPR
_TEMPLATECHOICE.fields_by_name['observers'].message_type = _EXPR
_TEMPLATECHOICE.fields_by_name['arg_binder'].message_type = _VARWITHTYPE
_TEMPLATECHOICE.fields_by_name['ret_type'].message_type = _TYPE
_TEMPLATECHOICE.fields_by_name['update'].message_type = _EXPR
_TEMPLATECHOICE.fields_by_name['location'].message_type = _LOCATION
_TEMPLATECHOICE.oneofs_by_name['name'].fields.append(
  _TEMPLATECHOICE.fields_by_name['name_str'])
_TEMPLATECHOICE.fields_by_name['name_str'].containing_oneof = _TEMPLATECHOICE.oneofs_by_name['name']
_TEMPLATECHOICE.oneofs_by_name['name'].fields.append(
  _TEMPLATECHOICE.fields_by_name['name_interned_str'])
_TEMPLATECHOICE.fields_by_name['name_interned_str'].containing_oneof = _TEMPLATECHOICE.oneofs_by_name['name']
_TEMPLATECHOICE.oneofs_by_name['self_binder'].fields.append(
  _TEMPLATECHOICE.fields_by_name['self_binder_str'])
_TEMPLATECHOICE.fields_by_name['self_binder_str'].containing_oneof = _TEMPLATECHOICE.oneofs_by_name['self_binder']
_TEMPLATECHOICE.oneofs_by_name['self_binder'].fields.append(
  _TEMPLATECHOICE.fields_by_name['self_binder_interned_str'])
_TEMPLATECHOICE.fields_by_name['self_binder_interned_str'].containing_oneof = _TEMPLATECHOICE.oneofs_by_name['self_binder']
_KEYEXPR_PROJECTION.fields_by_name['tycon'].message_type = _TYPE_CON
_KEYEXPR_PROJECTION.containing_type = _KEYEXPR
_KEYEXPR_PROJECTION.oneofs_by_name['field'].fields.append(
  _KEYEXPR_PROJECTION.fields_by_name['field_str'])
_KEYEXPR_PROJECTION.fields_by_name['field_str'].containing_oneof = _KEYEXPR_PROJECTION.oneofs_by_name['field']
_KEYEXPR_PROJECTION.oneofs_by_name['field'].fields.append(
  _KEYEXPR_PROJECTION.fields_by_name['field_interned_str'])
_KEYEXPR_PROJECTION.fields_by_name['field_interned_str'].containing_oneof = _KEYEXPR_PROJECTION.oneofs_by_name['field']
_KEYEXPR_PROJECTIONS.fields_by_name['projections'].message_type = _KEYEXPR_PROJECTION
_KEYEXPR_PROJECTIONS.containing_type = _KEYEXPR
_KEYEXPR_RECORDFIELD.fields_by_name['expr'].message_type = _KEYEXPR
_KEYEXPR_RECORDFIELD.containing_type = _KEYEXPR
_KEYEXPR_RECORDFIELD.oneofs_by_name['field'].fields.append(
  _KEYEXPR_RECORDFIELD.fields_by_name['field_str'])
_KEYEXPR_RECORDFIELD.fields_by_name['field_str'].containing_oneof = _KEYEXPR_RECORDFIELD.oneofs_by_name['field']
_KEYEXPR_RECORDFIELD.oneofs_by_name['field'].fields.append(
  _KEYEXPR_RECORDFIELD.fields_by_name['field_interned_str'])
_KEYEXPR_RECORDFIELD.fields_by_name['field_interned_str'].containing_oneof = _KEYEXPR_RECORDFIELD.oneofs_by_name['field']
_KEYEXPR_RECORD.fields_by_name['tycon'].message_type = _TYPE_CON
_KEYEXPR_RECORD.fields_by_name['fields'].message_type = _KEYEXPR_RECORDFIELD
_KEYEXPR_RECORD.containing_type = _KEYEXPR
_KEYEXPR.fields_by_name['projections'].message_type = _KEYEXPR_PROJECTIONS
_KEYEXPR.fields_by_name['record'].message_type = _KEYEXPR_RECORD
_KEYEXPR.oneofs_by_name['Sum'].fields.append(
  _KEYEXPR.fields_by_name['projections'])
_KEYEXPR.fields_by_name['projections'].containing_oneof = _KEYEXPR.oneofs_by_name['Sum']
_KEYEXPR.oneofs_by_name['Sum'].fields.append(
  _KEYEXPR.fields_by_name['record'])
_KEYEXPR.fields_by_name['record'].containing_oneof = _KEYEXPR.oneofs_by_name['Sum']
_DEFTEMPLATE_DEFKEY.fields_by_name['type'].message_type = _TYPE
_DEFTEMPLATE_DEFKEY.fields_by_name['key'].message_type = _KEYEXPR
_DEFTEMPLATE_DEFKEY.fields_by_name['complex_key'].message_type = _EXPR
_DEFTEMPLATE_DEFKEY.fields_by_name['maintainers'].message_type = _EXPR
_DEFTEMPLATE_DEFKEY.containing_type = _DEFTEMPLATE
_DEFTEMPLATE_DEFKEY.oneofs_by_name['key_expr'].fields.append(
  _DEFTEMPLATE_DEFKEY.fields_by_name['key'])
_DEFTEMPLATE_DEFKEY.fields_by_name['key'].containing_oneof = _DEFTEMPLATE_DEFKEY.oneofs_by_name['key_expr']
_DEFTEMPLATE_DEFKEY.oneofs_by_name['key_expr'].fields.append(
  _DEFTEMPLATE_DEFKEY.fields_by_name['complex_key'])
_DEFTEMPLATE_DEFKEY.fields_by_name['complex_key'].containing_oneof = _DEFTEMPLATE_DEFKEY.oneofs_by_name['key_expr']
_DEFTEMPLATE.fields_by_name['tycon_dname'].message_type = _DOTTEDNAME
_DEFTEMPLATE.fields_by_name['precond'].message_type = _EXPR
_DEFTEMPLATE.fields_by_name['signatories'].message_type = _EXPR
_DEFTEMPLATE.fields_by_name['agreement'].message_type = _EXPR
_DEFTEMPLATE.fields_by_name['choices'].message_type = _TEMPLATECHOICE
_DEFTEMPLATE.fields_by_name['observers'].message_type = _EXPR
_DEFTEMPLATE.fields_by_name['location'].message_type = _LOCATION
_DEFTEMPLATE.fields_by_name['key'].message_type = _DEFTEMPLATE_DEFKEY
_DEFTEMPLATE.oneofs_by_name['tycon'].fields.append(
  _DEFTEMPLATE.fields_by_name['tycon_dname'])
_DEFTEMPLATE.fields_by_name['tycon_dname'].containing_oneof = _DEFTEMPLATE.oneofs_by_name['tycon']
_DEFTEMPLATE.oneofs_by_name['tycon'].fields.append(
  _DEFTEMPLATE.fields_by_name['tycon_interned_dname'])
_DEFTEMPLATE.fields_by_name['tycon_interned_dname'].containing_oneof = _DEFTEMPLATE.oneofs_by_name['tycon']
_DEFTEMPLATE.oneofs_by_name['param'].fields.append(
  _DEFTEMPLATE.fields_by_name['param_str'])
_DEFTEMPLATE.fields_by_name['param_str'].containing_oneof = _DEFTEMPLATE.oneofs_by_name['param']
_DEFTEMPLATE.oneofs_by_name['param'].fields.append(
  _DEFTEMPLATE.fields_by_name['param_interned_str'])
_DEFTEMPLATE.fields_by_name['param_interned_str'].containing_oneof = _DEFTEMPLATE.oneofs_by_name['param']
_DEFEXCEPTION.fields_by_name['location'].message_type = _LOCATION
_DEFEXCEPTION.fields_by_name['message'].message_type = _EXPR
_DEFDATATYPE_FIELDS.fields_by_name['fields'].message_type = _FIELDWITHTYPE
_DEFDATATYPE_FIELDS.containing_type = _DEFDATATYPE
_DEFDATATYPE_ENUMCONSTRUCTORS.containing_type = _DEFDATATYPE
_DEFDATATYPE.fields_by_name['name_dname'].message_type = _DOTTEDNAME
_DEFDATATYPE.fields_by_name['params'].message_type = _TYPEVARWITHKIND
_DEFDATATYPE.fields_by_name['record'].message_type = _DEFDATATYPE_FIELDS
_DEFDATATYPE.fields_by_name['variant'].message_type = _DEFDATATYPE_FIELDS
_DEFDATATYPE.fields_by_name['enum'].message_type = _DEFDATATYPE_ENUMCONSTRUCTORS
_DEFDATATYPE.fields_by_name['location'].message_type = _LOCATION
_DEFDATATYPE.oneofs_by_name['name'].fields.append(
  _DEFDATATYPE.fields_by_name['name_dname'])
_DEFDATATYPE.fields_by_name['name_dname'].containing_oneof = _DEFDATATYPE.oneofs_by_name['name']
_DEFDATATYPE.oneofs_by_name['name'].fields.append(
  _DEFDATATYPE.fields_by_name['name_interned_dname'])
_DEFDATATYPE.fields_by_name['name_interned_dname'].containing_oneof = _DEFDATATYPE.oneofs_by_name['name']
_DEFDATATYPE.oneofs_by_name['DataCons'].fields.append(
  _DEFDATATYPE.fields_by_name['record'])
_DEFDATATYPE.fields_by_name['record'].containing_oneof = _DEFDATATYPE.oneofs_by_name['DataCons']
_DEFDATATYPE.oneofs_by_name['DataCons'].fields.append(
  _DEFDATATYPE.fields_by_name['variant'])
_DEFDATATYPE.fields_by_name['variant'].containing_oneof = _DEFDATATYPE.oneofs_by_name['DataCons']
_DEFDATATYPE.oneofs_by_name['DataCons'].fields.append(
  _DEFDATATYPE.fields_by_name['enum'])
_DEFDATATYPE.fields_by_name['enum'].containing_oneof = _DEFDATATYPE.oneofs_by_name['DataCons']
_DEFTYPESYN.fields_by_name['name_dname'].message_type = _DOTTEDNAME
_DEFTYPESYN.fields_by_name['params'].message_type = _TYPEVARWITHKIND
_DEFTYPESYN.fields_by_name['type'].message_type = _TYPE
_DEFTYPESYN.fields_by_name['location'].message_type = _LOCATION
_DEFTYPESYN.oneofs_by_name['name'].fields.append(
  _DEFTYPESYN.fields_by_name['name_dname'])
_DEFTYPESYN.fields_by_name['name_dname'].containing_oneof = _DEFTYPESYN.oneofs_by_name['name']
_DEFTYPESYN.oneofs_by_name['name'].fields.append(
  _DEFTYPESYN.fields_by_name['name_interned_dname'])
_DEFTYPESYN.fields_by_name['name_interned_dname'].containing_oneof = _DEFTYPESYN.oneofs_by_name['name']
_DEFVALUE_NAMEWITHTYPE.fields_by_name['type'].message_type = _TYPE
_DEFVALUE_NAMEWITHTYPE.containing_type = _DEFVALUE
_DEFVALUE.fields_by_name['name_with_type'].message_type = _DEFVALUE_NAMEWITHTYPE
_DEFVALUE.fields_by_name['expr'].message_type = _EXPR
_DEFVALUE.fields_by_name['location'].message_type = _LOCATION
_MODULE.fields_by_name['name_dname'].message_type = _DOTTEDNAME
_MODULE.fields_by_name['flags'].message_type = _FEATUREFLAGS
_MODULE.fields_by_name['synonyms'].message_type = _DEFTYPESYN
_MODULE.fields_by_name['data_types'].message_type = _DEFDATATYPE
_MODULE.fields_by_name['values'].message_type = _DEFVALUE
_MODULE.fields_by_name['templates'].message_type = _DEFTEMPLATE
_MODULE.fields_by_name['exceptions'].message_type = _DEFEXCEPTION
_MODULE.oneofs_by_name['name'].fields.append(
  _MODULE.fields_by_name['name_dname'])
_MODULE.fields_by_name['name_dname'].containing_oneof = _MODULE.oneofs_by_name['name']
_MODULE.oneofs_by_name['name'].fields.append(
  _MODULE.fields_by_name['name_interned_dname'])
_MODULE.fields_by_name['name_interned_dname'].containing_oneof = _MODULE.oneofs_by_name['name']
_PACKAGE.fields_by_name['modules'].message_type = _MODULE
_PACKAGE.fields_by_name['interned_dotted_names'].message_type = _INTERNEDDOTTEDNAME
_PACKAGE.fields_by_name['metadata'].message_type = _PACKAGEMETADATA
_PACKAGE.fields_by_name['interned_types'].message_type = _TYPE
DESCRIPTOR.message_types_by_name['Unit'] = _UNIT
DESCRIPTOR.message_types_by_name['PackageRef'] = _PACKAGEREF
DESCRIPTOR.message_types_by_name['DottedName'] = _DOTTEDNAME
DESCRIPTOR.message_types_by_name['ModuleRef'] = _MODULEREF
DESCRIPTOR.message_types_by_name['TypeConName'] = _TYPECONNAME
DESCRIPTOR.message_types_by_name['TypeSynName'] = _TYPESYNNAME
DESCRIPTOR.message_types_by_name['ValName'] = _VALNAME
DESCRIPTOR.message_types_by_name['FieldWithType'] = _FIELDWITHTYPE
DESCRIPTOR.message_types_by_name['VarWithType'] = _VARWITHTYPE
DESCRIPTOR.message_types_by_name['TypeVarWithKind'] = _TYPEVARWITHKIND
DESCRIPTOR.message_types_by_name['FieldWithExpr'] = _FIELDWITHEXPR
DESCRIPTOR.message_types_by_name['Binding'] = _BINDING
DESCRIPTOR.message_types_by_name['Kind'] = _KIND
DESCRIPTOR.message_types_by_name['Type'] = _TYPE
DESCRIPTOR.message_types_by_name['PrimLit'] = _PRIMLIT
DESCRIPTOR.message_types_by_name['Location'] = _LOCATION
DESCRIPTOR.message_types_by_name['Expr'] = _EXPR
DESCRIPTOR.message_types_by_name['CaseAlt'] = _CASEALT
DESCRIPTOR.message_types_by_name['Case'] = _CASE
DESCRIPTOR.message_types_by_name['Block'] = _BLOCK
DESCRIPTOR.message_types_by_name['Pure'] = _PURE
DESCRIPTOR.message_types_by_name['Update'] = _UPDATE
DESCRIPTOR.message_types_by_name['Scenario'] = _SCENARIO
DESCRIPTOR.message_types_by_name['TemplateChoice'] = _TEMPLATECHOICE
DESCRIPTOR.message_types_by_name['KeyExpr'] = _KEYEXPR
DESCRIPTOR.message_types_by_name['DefTemplate'] = _DEFTEMPLATE
DESCRIPTOR.message_types_by_name['DefException'] = _DEFEXCEPTION
DESCRIPTOR.message_types_by_name['DefDataType'] = _DEFDATATYPE
DESCRIPTOR.message_types_by_name['DefTypeSyn'] = _DEFTYPESYN
DESCRIPTOR.message_types_by_name['DefValue'] = _DEFVALUE
DESCRIPTOR.message_types_by_name['FeatureFlags'] = _FEATUREFLAGS
DESCRIPTOR.message_types_by_name['Module'] = _MODULE
DESCRIPTOR.message_types_by_name['InternedDottedName'] = _INTERNEDDOTTEDNAME
DESCRIPTOR.message_types_by_name['PackageMetadata'] = _PACKAGEMETADATA
DESCRIPTOR.message_types_by_name['Package'] = _PACKAGE
DESCRIPTOR.enum_types_by_name['PrimType'] = _PRIMTYPE
DESCRIPTOR.enum_types_by_name['PrimCon'] = _PRIMCON
DESCRIPTOR.enum_types_by_name['BuiltinFunction'] = _BUILTINFUNCTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Unit = _reflection.GeneratedProtocolMessageType('Unit', (_message.Message,), {
  'DESCRIPTOR' : _UNIT,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Unit)
  })
_sym_db.RegisterMessage(Unit)

PackageRef = _reflection.GeneratedProtocolMessageType('PackageRef', (_message.Message,), {
  'DESCRIPTOR' : _PACKAGEREF,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.PackageRef)
  })
_sym_db.RegisterMessage(PackageRef)

DottedName = _reflection.GeneratedProtocolMessageType('DottedName', (_message.Message,), {
  'DESCRIPTOR' : _DOTTEDNAME,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.DottedName)
  })
_sym_db.RegisterMessage(DottedName)

ModuleRef = _reflection.GeneratedProtocolMessageType('ModuleRef', (_message.Message,), {
  'DESCRIPTOR' : _MODULEREF,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.ModuleRef)
  })
_sym_db.RegisterMessage(ModuleRef)

TypeConName = _reflection.GeneratedProtocolMessageType('TypeConName', (_message.Message,), {
  'DESCRIPTOR' : _TYPECONNAME,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.TypeConName)
  })
_sym_db.RegisterMessage(TypeConName)

TypeSynName = _reflection.GeneratedProtocolMessageType('TypeSynName', (_message.Message,), {
  'DESCRIPTOR' : _TYPESYNNAME,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.TypeSynName)
  })
_sym_db.RegisterMessage(TypeSynName)

ValName = _reflection.GeneratedProtocolMessageType('ValName', (_message.Message,), {
  'DESCRIPTOR' : _VALNAME,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.ValName)
  })
_sym_db.RegisterMessage(ValName)

FieldWithType = _reflection.GeneratedProtocolMessageType('FieldWithType', (_message.Message,), {
  'DESCRIPTOR' : _FIELDWITHTYPE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.FieldWithType)
  })
_sym_db.RegisterMessage(FieldWithType)

VarWithType = _reflection.GeneratedProtocolMessageType('VarWithType', (_message.Message,), {
  'DESCRIPTOR' : _VARWITHTYPE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.VarWithType)
  })
_sym_db.RegisterMessage(VarWithType)

TypeVarWithKind = _reflection.GeneratedProtocolMessageType('TypeVarWithKind', (_message.Message,), {
  'DESCRIPTOR' : _TYPEVARWITHKIND,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.TypeVarWithKind)
  })
_sym_db.RegisterMessage(TypeVarWithKind)

FieldWithExpr = _reflection.GeneratedProtocolMessageType('FieldWithExpr', (_message.Message,), {
  'DESCRIPTOR' : _FIELDWITHEXPR,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.FieldWithExpr)
  })
_sym_db.RegisterMessage(FieldWithExpr)

Binding = _reflection.GeneratedProtocolMessageType('Binding', (_message.Message,), {
  'DESCRIPTOR' : _BINDING,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Binding)
  })
_sym_db.RegisterMessage(Binding)

Kind = _reflection.GeneratedProtocolMessageType('Kind', (_message.Message,), {

  'Arrow' : _reflection.GeneratedProtocolMessageType('Arrow', (_message.Message,), {
    'DESCRIPTOR' : _KIND_ARROW,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Kind.Arrow)
    })
  ,
  'DESCRIPTOR' : _KIND,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Kind)
  })
_sym_db.RegisterMessage(Kind)
_sym_db.RegisterMessage(Kind.Arrow)

Type = _reflection.GeneratedProtocolMessageType('Type', (_message.Message,), {

  'Var' : _reflection.GeneratedProtocolMessageType('Var', (_message.Message,), {
    'DESCRIPTOR' : _TYPE_VAR,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Type.Var)
    })
  ,

  'Con' : _reflection.GeneratedProtocolMessageType('Con', (_message.Message,), {
    'DESCRIPTOR' : _TYPE_CON,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Type.Con)
    })
  ,

  'Syn' : _reflection.GeneratedProtocolMessageType('Syn', (_message.Message,), {
    'DESCRIPTOR' : _TYPE_SYN,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Type.Syn)
    })
  ,

  'Prim' : _reflection.GeneratedProtocolMessageType('Prim', (_message.Message,), {
    'DESCRIPTOR' : _TYPE_PRIM,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Type.Prim)
    })
  ,

  'Fun' : _reflection.GeneratedProtocolMessageType('Fun', (_message.Message,), {
    'DESCRIPTOR' : _TYPE_FUN,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Type.Fun)
    })
  ,

  'Forall' : _reflection.GeneratedProtocolMessageType('Forall', (_message.Message,), {
    'DESCRIPTOR' : _TYPE_FORALL,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Type.Forall)
    })
  ,

  'Struct' : _reflection.GeneratedProtocolMessageType('Struct', (_message.Message,), {
    'DESCRIPTOR' : _TYPE_STRUCT,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Type.Struct)
    })
  ,
  'DESCRIPTOR' : _TYPE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Type)
  })
_sym_db.RegisterMessage(Type)
_sym_db.RegisterMessage(Type.Var)
_sym_db.RegisterMessage(Type.Con)
_sym_db.RegisterMessage(Type.Syn)
_sym_db.RegisterMessage(Type.Prim)
_sym_db.RegisterMessage(Type.Fun)
_sym_db.RegisterMessage(Type.Forall)
_sym_db.RegisterMessage(Type.Struct)

PrimLit = _reflection.GeneratedProtocolMessageType('PrimLit', (_message.Message,), {
  'DESCRIPTOR' : _PRIMLIT,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.PrimLit)
  })
_sym_db.RegisterMessage(PrimLit)

Location = _reflection.GeneratedProtocolMessageType('Location', (_message.Message,), {

  'Range' : _reflection.GeneratedProtocolMessageType('Range', (_message.Message,), {
    'DESCRIPTOR' : _LOCATION_RANGE,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Location.Range)
    })
  ,
  'DESCRIPTOR' : _LOCATION,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Location)
  })
_sym_db.RegisterMessage(Location)
_sym_db.RegisterMessage(Location.Range)

Expr = _reflection.GeneratedProtocolMessageType('Expr', (_message.Message,), {

  'RecCon' : _reflection.GeneratedProtocolMessageType('RecCon', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_RECCON,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.RecCon)
    })
  ,

  'RecProj' : _reflection.GeneratedProtocolMessageType('RecProj', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_RECPROJ,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.RecProj)
    })
  ,

  'RecUpd' : _reflection.GeneratedProtocolMessageType('RecUpd', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_RECUPD,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.RecUpd)
    })
  ,

  'VariantCon' : _reflection.GeneratedProtocolMessageType('VariantCon', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_VARIANTCON,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.VariantCon)
    })
  ,

  'EnumCon' : _reflection.GeneratedProtocolMessageType('EnumCon', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_ENUMCON,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.EnumCon)
    })
  ,

  'StructCon' : _reflection.GeneratedProtocolMessageType('StructCon', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_STRUCTCON,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.StructCon)
    })
  ,

  'StructProj' : _reflection.GeneratedProtocolMessageType('StructProj', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_STRUCTPROJ,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.StructProj)
    })
  ,

  'StructUpd' : _reflection.GeneratedProtocolMessageType('StructUpd', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_STRUCTUPD,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.StructUpd)
    })
  ,

  'App' : _reflection.GeneratedProtocolMessageType('App', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_APP,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.App)
    })
  ,

  'TyApp' : _reflection.GeneratedProtocolMessageType('TyApp', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_TYAPP,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.TyApp)
    })
  ,

  'Abs' : _reflection.GeneratedProtocolMessageType('Abs', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_ABS,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.Abs)
    })
  ,

  'TyAbs' : _reflection.GeneratedProtocolMessageType('TyAbs', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_TYABS,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.TyAbs)
    })
  ,

  'Nil' : _reflection.GeneratedProtocolMessageType('Nil', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_NIL,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.Nil)
    })
  ,

  'Cons' : _reflection.GeneratedProtocolMessageType('Cons', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_CONS,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.Cons)
    })
  ,

  'OptionalNone' : _reflection.GeneratedProtocolMessageType('OptionalNone', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_OPTIONALNONE,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.OptionalNone)
    })
  ,

  'OptionalSome' : _reflection.GeneratedProtocolMessageType('OptionalSome', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_OPTIONALSOME,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.OptionalSome)
    })
  ,

  'ToAny' : _reflection.GeneratedProtocolMessageType('ToAny', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_TOANY,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.ToAny)
    })
  ,

  'FromAny' : _reflection.GeneratedProtocolMessageType('FromAny', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_FROMANY,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.FromAny)
    })
  ,

  'ToAnyException' : _reflection.GeneratedProtocolMessageType('ToAnyException', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_TOANYEXCEPTION,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.ToAnyException)
    })
  ,

  'FromAnyException' : _reflection.GeneratedProtocolMessageType('FromAnyException', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_FROMANYEXCEPTION,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.FromAnyException)
    })
  ,

  'Throw' : _reflection.GeneratedProtocolMessageType('Throw', (_message.Message,), {
    'DESCRIPTOR' : _EXPR_THROW,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Expr.Throw)
    })
  ,
  'DESCRIPTOR' : _EXPR,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Expr)
  })
_sym_db.RegisterMessage(Expr)
_sym_db.RegisterMessage(Expr.RecCon)
_sym_db.RegisterMessage(Expr.RecProj)
_sym_db.RegisterMessage(Expr.RecUpd)
_sym_db.RegisterMessage(Expr.VariantCon)
_sym_db.RegisterMessage(Expr.EnumCon)
_sym_db.RegisterMessage(Expr.StructCon)
_sym_db.RegisterMessage(Expr.StructProj)
_sym_db.RegisterMessage(Expr.StructUpd)
_sym_db.RegisterMessage(Expr.App)
_sym_db.RegisterMessage(Expr.TyApp)
_sym_db.RegisterMessage(Expr.Abs)
_sym_db.RegisterMessage(Expr.TyAbs)
_sym_db.RegisterMessage(Expr.Nil)
_sym_db.RegisterMessage(Expr.Cons)
_sym_db.RegisterMessage(Expr.OptionalNone)
_sym_db.RegisterMessage(Expr.OptionalSome)
_sym_db.RegisterMessage(Expr.ToAny)
_sym_db.RegisterMessage(Expr.FromAny)
_sym_db.RegisterMessage(Expr.ToAnyException)
_sym_db.RegisterMessage(Expr.FromAnyException)
_sym_db.RegisterMessage(Expr.Throw)

CaseAlt = _reflection.GeneratedProtocolMessageType('CaseAlt', (_message.Message,), {

  'Variant' : _reflection.GeneratedProtocolMessageType('Variant', (_message.Message,), {
    'DESCRIPTOR' : _CASEALT_VARIANT,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.CaseAlt.Variant)
    })
  ,

  'Enum' : _reflection.GeneratedProtocolMessageType('Enum', (_message.Message,), {
    'DESCRIPTOR' : _CASEALT_ENUM,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.CaseAlt.Enum)
    })
  ,

  'Cons' : _reflection.GeneratedProtocolMessageType('Cons', (_message.Message,), {
    'DESCRIPTOR' : _CASEALT_CONS,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.CaseAlt.Cons)
    })
  ,

  'OptionalSome' : _reflection.GeneratedProtocolMessageType('OptionalSome', (_message.Message,), {
    'DESCRIPTOR' : _CASEALT_OPTIONALSOME,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.CaseAlt.OptionalSome)
    })
  ,
  'DESCRIPTOR' : _CASEALT,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.CaseAlt)
  })
_sym_db.RegisterMessage(CaseAlt)
_sym_db.RegisterMessage(CaseAlt.Variant)
_sym_db.RegisterMessage(CaseAlt.Enum)
_sym_db.RegisterMessage(CaseAlt.Cons)
_sym_db.RegisterMessage(CaseAlt.OptionalSome)

Case = _reflection.GeneratedProtocolMessageType('Case', (_message.Message,), {
  'DESCRIPTOR' : _CASE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Case)
  })
_sym_db.RegisterMessage(Case)

Block = _reflection.GeneratedProtocolMessageType('Block', (_message.Message,), {
  'DESCRIPTOR' : _BLOCK,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Block)
  })
_sym_db.RegisterMessage(Block)

Pure = _reflection.GeneratedProtocolMessageType('Pure', (_message.Message,), {
  'DESCRIPTOR' : _PURE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Pure)
  })
_sym_db.RegisterMessage(Pure)

Update = _reflection.GeneratedProtocolMessageType('Update', (_message.Message,), {

  'Create' : _reflection.GeneratedProtocolMessageType('Create', (_message.Message,), {
    'DESCRIPTOR' : _UPDATE_CREATE,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Update.Create)
    })
  ,

  'Exercise' : _reflection.GeneratedProtocolMessageType('Exercise', (_message.Message,), {
    'DESCRIPTOR' : _UPDATE_EXERCISE,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Update.Exercise)
    })
  ,

  'ExerciseByKey' : _reflection.GeneratedProtocolMessageType('ExerciseByKey', (_message.Message,), {
    'DESCRIPTOR' : _UPDATE_EXERCISEBYKEY,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Update.ExerciseByKey)
    })
  ,

  'Fetch' : _reflection.GeneratedProtocolMessageType('Fetch', (_message.Message,), {
    'DESCRIPTOR' : _UPDATE_FETCH,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Update.Fetch)
    })
  ,

  'EmbedExpr' : _reflection.GeneratedProtocolMessageType('EmbedExpr', (_message.Message,), {
    'DESCRIPTOR' : _UPDATE_EMBEDEXPR,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Update.EmbedExpr)
    })
  ,

  'RetrieveByKey' : _reflection.GeneratedProtocolMessageType('RetrieveByKey', (_message.Message,), {
    'DESCRIPTOR' : _UPDATE_RETRIEVEBYKEY,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Update.RetrieveByKey)
    })
  ,

  'TryCatch' : _reflection.GeneratedProtocolMessageType('TryCatch', (_message.Message,), {
    'DESCRIPTOR' : _UPDATE_TRYCATCH,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Update.TryCatch)
    })
  ,
  'DESCRIPTOR' : _UPDATE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Update)
  })
_sym_db.RegisterMessage(Update)
_sym_db.RegisterMessage(Update.Create)
_sym_db.RegisterMessage(Update.Exercise)
_sym_db.RegisterMessage(Update.ExerciseByKey)
_sym_db.RegisterMessage(Update.Fetch)
_sym_db.RegisterMessage(Update.EmbedExpr)
_sym_db.RegisterMessage(Update.RetrieveByKey)
_sym_db.RegisterMessage(Update.TryCatch)

Scenario = _reflection.GeneratedProtocolMessageType('Scenario', (_message.Message,), {

  'Commit' : _reflection.GeneratedProtocolMessageType('Commit', (_message.Message,), {
    'DESCRIPTOR' : _SCENARIO_COMMIT,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Scenario.Commit)
    })
  ,

  'EmbedExpr' : _reflection.GeneratedProtocolMessageType('EmbedExpr', (_message.Message,), {
    'DESCRIPTOR' : _SCENARIO_EMBEDEXPR,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.Scenario.EmbedExpr)
    })
  ,
  'DESCRIPTOR' : _SCENARIO,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Scenario)
  })
_sym_db.RegisterMessage(Scenario)
_sym_db.RegisterMessage(Scenario.Commit)
_sym_db.RegisterMessage(Scenario.EmbedExpr)

TemplateChoice = _reflection.GeneratedProtocolMessageType('TemplateChoice', (_message.Message,), {
  'DESCRIPTOR' : _TEMPLATECHOICE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.TemplateChoice)
  })
_sym_db.RegisterMessage(TemplateChoice)

KeyExpr = _reflection.GeneratedProtocolMessageType('KeyExpr', (_message.Message,), {

  'Projection' : _reflection.GeneratedProtocolMessageType('Projection', (_message.Message,), {
    'DESCRIPTOR' : _KEYEXPR_PROJECTION,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.KeyExpr.Projection)
    })
  ,

  'Projections' : _reflection.GeneratedProtocolMessageType('Projections', (_message.Message,), {
    'DESCRIPTOR' : _KEYEXPR_PROJECTIONS,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.KeyExpr.Projections)
    })
  ,

  'RecordField' : _reflection.GeneratedProtocolMessageType('RecordField', (_message.Message,), {
    'DESCRIPTOR' : _KEYEXPR_RECORDFIELD,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.KeyExpr.RecordField)
    })
  ,

  'Record' : _reflection.GeneratedProtocolMessageType('Record', (_message.Message,), {
    'DESCRIPTOR' : _KEYEXPR_RECORD,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.KeyExpr.Record)
    })
  ,
  'DESCRIPTOR' : _KEYEXPR,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.KeyExpr)
  })
_sym_db.RegisterMessage(KeyExpr)
_sym_db.RegisterMessage(KeyExpr.Projection)
_sym_db.RegisterMessage(KeyExpr.Projections)
_sym_db.RegisterMessage(KeyExpr.RecordField)
_sym_db.RegisterMessage(KeyExpr.Record)

DefTemplate = _reflection.GeneratedProtocolMessageType('DefTemplate', (_message.Message,), {

  'DefKey' : _reflection.GeneratedProtocolMessageType('DefKey', (_message.Message,), {
    'DESCRIPTOR' : _DEFTEMPLATE_DEFKEY,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.DefTemplate.DefKey)
    })
  ,
  'DESCRIPTOR' : _DEFTEMPLATE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.DefTemplate)
  })
_sym_db.RegisterMessage(DefTemplate)
_sym_db.RegisterMessage(DefTemplate.DefKey)

DefException = _reflection.GeneratedProtocolMessageType('DefException', (_message.Message,), {
  'DESCRIPTOR' : _DEFEXCEPTION,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.DefException)
  })
_sym_db.RegisterMessage(DefException)

DefDataType = _reflection.GeneratedProtocolMessageType('DefDataType', (_message.Message,), {

  'Fields' : _reflection.GeneratedProtocolMessageType('Fields', (_message.Message,), {
    'DESCRIPTOR' : _DEFDATATYPE_FIELDS,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.DefDataType.Fields)
    })
  ,

  'EnumConstructors' : _reflection.GeneratedProtocolMessageType('EnumConstructors', (_message.Message,), {
    'DESCRIPTOR' : _DEFDATATYPE_ENUMCONSTRUCTORS,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.DefDataType.EnumConstructors)
    })
  ,
  'DESCRIPTOR' : _DEFDATATYPE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.DefDataType)
  })
_sym_db.RegisterMessage(DefDataType)
_sym_db.RegisterMessage(DefDataType.Fields)
_sym_db.RegisterMessage(DefDataType.EnumConstructors)

DefTypeSyn = _reflection.GeneratedProtocolMessageType('DefTypeSyn', (_message.Message,), {
  'DESCRIPTOR' : _DEFTYPESYN,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.DefTypeSyn)
  })
_sym_db.RegisterMessage(DefTypeSyn)

DefValue = _reflection.GeneratedProtocolMessageType('DefValue', (_message.Message,), {

  'NameWithType' : _reflection.GeneratedProtocolMessageType('NameWithType', (_message.Message,), {
    'DESCRIPTOR' : _DEFVALUE_NAMEWITHTYPE,
    '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
    # @@protoc_insertion_point(class_scope:daml_lf_1.DefValue.NameWithType)
    })
  ,
  'DESCRIPTOR' : _DEFVALUE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.DefValue)
  })
_sym_db.RegisterMessage(DefValue)
_sym_db.RegisterMessage(DefValue.NameWithType)

FeatureFlags = _reflection.GeneratedProtocolMessageType('FeatureFlags', (_message.Message,), {
  'DESCRIPTOR' : _FEATUREFLAGS,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.FeatureFlags)
  })
_sym_db.RegisterMessage(FeatureFlags)

Module = _reflection.GeneratedProtocolMessageType('Module', (_message.Message,), {
  'DESCRIPTOR' : _MODULE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Module)
  })
_sym_db.RegisterMessage(Module)

InternedDottedName = _reflection.GeneratedProtocolMessageType('InternedDottedName', (_message.Message,), {
  'DESCRIPTOR' : _INTERNEDDOTTEDNAME,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.InternedDottedName)
  })
_sym_db.RegisterMessage(InternedDottedName)

PackageMetadata = _reflection.GeneratedProtocolMessageType('PackageMetadata', (_message.Message,), {
  'DESCRIPTOR' : _PACKAGEMETADATA,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.PackageMetadata)
  })
_sym_db.RegisterMessage(PackageMetadata)

Package = _reflection.GeneratedProtocolMessageType('Package', (_message.Message,), {
  'DESCRIPTOR' : _PACKAGE,
  '__module__' : 'com.daml.daml_lf_dev.daml_lf_1_pb2'
  # @@protoc_insertion_point(class_scope:daml_lf_1.Package)
  })
_sym_db.RegisterMessage(Package)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
